<?php

namespace Teknisa\FIS\Modules\Apuracoes\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Apuracoes\Factories\Fis00100ApuracaoDeImpostoFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Masks\Masks;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Model\Fiscal\Entities\Mapaapurac;
use Teknisa\Model\Fiscal\Entities\Mapaapuracsn;
use Zeedhi\Framework\DTO\Row;
use DateTime;

class Fis00100ApuracaoDeImpostoService {

    const DOCUMENTOS_FORA_PERIODO        = "NF_F_PER";
    const FAIXA_NAO_CADASTRADA           = "Faixa_NCad";
    const TABELA_SIMPLES_NAO_CADASTRADA  = "TabelaSimples_NCad";
    const IMPOSTOS_CONTRIBUICOES         = array("50","60");
    const IMPOSTOS_INSS                  = "42";
    const IMPOSTOS_IRPJCSLL              = array("25","70");
    const IMPOSTO_ICMSST                 = "80";

    protected $dataSourceManager;
    protected $entityManager;
    protected $environment;
    protected $factoryApuracaoDeImposto;
    protected $masks;
    protected $newCode;
    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $crudService;
    
    public function __construct($crudService, $dataSourceManager, \Doctrine\ORM\EntityManager $entityManager, Environment $environment, NewCode $newCode ,Fis00100ApuracaoDeImpostoFactory $factoryApuracaoDeImposto, Masks $masks) {
        $this->crudService              = $crudService;
        $this->dataSourceManager        = $dataSourceManager;
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->factoryApuracaoDeImposto = $factoryApuracaoDeImposto;
        $this->masks                    = $masks;
        $this->newCode                  = $newCode;
        $this->nrorg                    = $environment->getNrorgTrab();
        $this->operador                 = $this->environment->getCdOperador();
        $this->currentTime              = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }


    #############################################################
    #                                                           #
    #      Funções para manipular dados e recontabilização      #
    #                                                           #
    #############################################################

    /*
     * Função para recontabilizar todos os valores do detalhamento
     de um mapa do Inss 
     */
    public function calculaDetalhamentoIcmsSt ($row){
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $maparecup        = $this->factoryApuracaoDeImposto->getMaparecup($params);
        $mapaapurac       = $this->factoryApuracaoDeImposto->getMapaapurac($params);
        $mapaapurestado   = $this->factoryApuracaoDeImposto->getMapaapurestado(
            array(
                "NRSEQUMAPAPU"     => $row["NRSEQUMAPAPU"],
                "NRORG"            => $this->nrorg,
                "NRSEQMAPAAPUREST" => $row["NRSEQMAPAAPUREST"],
                "SGESTADO"         => $row["SGESTADO"],
                "CDPAIS"           => $row["CDPAIS"]
            )    
        );
        
        $somaDetalhamento = $this->factoryApuracaoDeImposto->getSomasDetalhamentoIcmsSt(
            array(
                "NRSEQUMAPAPU"     => $row["NRSEQUMAPAPU"],
                "NRORG"            => $this->nrorg,
                "NRSEQMAPAAPUREST" => $row["NRSEQMAPAAPUREST"],
                "SGESTADO"         => $row["SGESTADO"],
                "CDPAIS"           => $row["CDPAIS"]
            )  
        );
        
        $mapaapurac['__is_new']     = false;
        $maparecup['__is_new']      = false;
        $mapaapurestado['__is_new'] = false;
        
        
        $mapaapurestado['VROUTROSCREDITOS'] = $somaDetalhamento['VROUTROSCREDITOS'];
        $mapaapurestado['VROUTROSDEBITOS']  = $somaDetalhamento['VROUTROSDEBITOS'];
    
        //Total Créditos: Crédito Normal + Outros Créditos + Ressarcimento + Devolução
        $mapaapurestado['VRTOTACRED'] = (float) $mapaapurestado['VRCREDITONORMAL'] + (float) $mapaapurestado['VROUTROSCREDITOS'] + (float) $mapaapurestado['VRRESSAMAPA'] + (float) $mapaapurestado['VRDEVMAPA'] ;  
        //Total Débitos: Débito Normal + Outros Débitos 
        $mapaapurestado['VRTOTADEBI'] = (float) $mapaapurestado['VRDEBITONORMAL']  + (float) $mapaapurestado['VROUTROSDEBITOS'];  
        
        $mapaapurestado['VRDEDUZFECP'] = $somaDetalhamento['VRDEDUZFECP'];
        
        $mapaapurestado['VRDEBESPFECP'] = $somaDetalhamento['VRDEBESPFECP'];
        
    
        
        /* Se Total de Débitos - Total Créditos – Crédito Anterior  < 0
        Saldo Credor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções
        Saldo Devedor: 0 */
        $conta = (float)$mapaapurestado['VRTOTADEBI'] - (float)$mapaapurestado['VRTOTACRED'] - (float)$mapaapurestado['VRCREDANTERIOR'] - $mapaapurestado['VRDEDUZFECP'];
        if($conta < 0 ){
            $mapaapurestado['VRSALDOCREDOR']  = abs($conta); // valor absoluta da conta
            $mapaapurestado['VRIMPARECOLHER'] = 0;
        }
        /*Se Não
            Saldo Credor: 0
            Saldo Devedor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções */
        else{
            $mapaapurestado['VRSALDOCREDOR']  = 0; // valor absoluta da conta
            $mapaapurestado['VRIMPARECOLHER'] = abs($conta); // valor absoluta da conta
        }
        
        $this->saveGeral($mapaapurestado,'mapaapurestado');
        $this->entityManager->flush();
        
        
        $somaValorImposto          = $this->factoryApuracaoDeImposto->getSomaImpostosIcmsSt($params);
        $mapaapurac['VRTOTAIMPOS'] = $somaValorImposto['SOMAIMPOSTO'];
        
        $maparecup['VRSALDODEBI'] = $somaValorImposto['SOMAIMPOSTO']; 
        $maparecup['VRSALDOCRED'] = $somaValorImposto['SOMACREDOR'];


        $this->saveGeral($maparecup,'maparecup');
        $this->entityManager->flush();
        
        $this->saveGeral($mapaapurac,'mapaapurac');
        $this->entityManager->flush();
        
    }
    
    /*
     * Função para recontabilizar todos os valores do detalhamento
     de um mapa do Inss 
     */
    public function calculaDetalhamentoInss ($row){
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $mapaapurac       = $this->factoryApuracaoDeImposto->getMapaapurac($params);
        $somaDetalhamento = $this->factoryApuracaoDeImposto->getSomasItensDetalhamentoContribuicoes($params);
        
        $mapaapurac ['__is_new']    = false;
        $mapaapurac ['VRCOMPENSA']  = $somaDetalhamento['VRCOMPENSA'];
        $conta                      =  (float) $mapaapurac['VRTTCTBLDEB'] - (float) $mapaapurac['VRCOMPENSA'];
        
        $mapaapurac ['VRTOTAIMPOS'] = $conta>0 ? $conta : 0;
        
        $this->saveGeral($mapaapurac,'mapaapurac');
    }

    /*
     * Função para recontabilizar todos os valores de um mapa dd
     * ICMS referente a aba do detalhamento DIFAL/FCP
     */
    public function calculaDetalhamentoDifalFcp ($row){
        $params = array(
            "NRSEQMAPAPURESTNC" => $row["NRSEQMAPAPURESTNC"],
            "NRORG"             => $this->nrorg
        );
        
        $mapaapurestadonc = $this->factoryApuracaoDeImposto->getMapaapurestadonc($params);
        $maparecupnc      = $this->factoryApuracaoDeImposto->getMaparecupnc($params);
        $maparecup        = $this->factoryApuracaoDeImposto->getMaparecup(
            array(
                "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                "NRORG" => $this->nrorg
            )
        );
        
        //Prepara para edição dos valores
        $maparecupnc      ['__is_new']   = false;
        $mapaapurestadonc ['__is_new']   = false;
        $maparecup        ['__is_new']   = false;
        
        $somaDetalhamento = $this->factoryApuracaoDeImposto->getSomasDetalhamentoIcmsDifalFcp(
            array(
                "P_NRSEQUMAPAPU"      => $mapaapurestadonc['NRSEQUMAPAPU'],
                "P_NRSEQMAPAPURESTNC" => $mapaapurestadonc['NRSEQMAPAPURESTNC'],
                "P_IDAJUSTENC"        => $row['IDAJUSTENC'],
                "P_NRORG"             => $this->nrorg,
                
            )    
        );
       
        //Caso esteja atualizando o detalhamento do DIFAL
        if ($row['IDAJUSTENC'] ==='D'){
            $maparecupnc['VROUTRDEBIDIFAL'] = $somaDetalhamento['VROUTROSDEBITOS'];
            $maparecupnc['VRESTOCREDDIFAL'] = $somaDetalhamento['VRESTORNOCREDITO'];
            $maparecupnc['VROUTRCREDDIFAL'] = $somaDetalhamento['VROUTROSCREDITOS'];
            $maparecupnc['VRESTODEBIDIFAL'] = $somaDetalhamento['VRESTORNODEBITO'];
            $maparecupnc['VRDEDUCAODIFAL']  = $somaDetalhamento['VRDEDUCOES'];
            
             //Total Créditos: Crédito DIFAL + Outros Créditos + Estorno Débitos
            $maparecupnc['VRTOTACREDDIFAL'] = (float) $maparecupnc['VRCREDDIFAL'] + (float) $maparecupnc['VROUTRCREDDIFAL'] + (float) $maparecupnc['VRESTODEBIDIFAL'];
        
            //Total Débitos: Débito DIFAL + Outros Débitos + Estorno Crédito
            $maparecupnc['VRTOTADEBIDIFAL'] = (float) $maparecupnc['VRDEBIDIFAL'] + (float) $maparecupnc['VROUTRDEBIDIFAL'] + (float) $maparecupnc['VRESTOCREDDIFAL'];
        
            
            /* Se Total de Débitos - Total Créditos – Crédito Anterior – Deduções < 0
            Saldo Credor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções
            Saldo Devedor: 0 */
            $conta = (float)$maparecupnc['VRTOTADEBIDIFAL'] - (float)$maparecupnc['VRTOTACREDDIFAL'] - (float)$maparecupnc['VRSLDCRANTEDIFAL'] - (float)$maparecupnc['VRDEDUCAODIFAL'];
            if($conta < 0 ){
                $maparecupnc['VRSALDOCREDDIFAL']   = abs($conta); // valor absoluta da conta
                $maparecupnc['VRSALDODEVEDIFAL']   = 0;
                $mapaapurestadonc['VRIMPRECDIFAL'] = 0;
                
                
            }
            /*Se Não
                Saldo Credor: 0
                Saldo Devedor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções */
            else{
                $maparecupnc['VRSALDOCREDDIFAL']   = 0; // valor absoluta da conta
                $maparecupnc['VRSALDODEVEDIFAL']   = abs($conta); // valor absoluta da conta
                $mapaapurestadonc['VRIMPRECDIFAL'] = abs($conta);
            }
            
            
        }
        //Caso esteja atualizando o detalhamento do FCP
        else if ($row['IDAJUSTENC'] ==='F')  {
            $maparecupnc['VROUTRDEBIFCP'] = $somaDetalhamento['VROUTROSDEBITOS'];
            $maparecupnc['VRESTOCREDFCP'] = $somaDetalhamento['VRESTORNOCREDITO'];
            $maparecupnc['VROUTRCREDFCP'] = $somaDetalhamento['VROUTROSCREDITOS'];
            $maparecupnc['VRESTODEBIFCP'] = $somaDetalhamento['VRESTORNODEBITO'];
            $maparecupnc['VRDEDUCAOFCP']  = $somaDetalhamento['VRDEDUCOES'];
            
            //Total Créditos: Crédito FCP + Outros Créditos + Estorno Débitos
            $maparecupnc['VRTOTACREDFCP'] = (float) $maparecupnc['VRCREDFCP'] + (float) $maparecupnc['VROUTRCREDFCP'] + (float) $maparecupnc['VRESTODEBIFCP'];
        
            //Total Débitos: Débito FCP + Outros Débitos + Estorno Crédito
            $maparecupnc['VRTOTADEBIFCP'] = (float) $maparecupnc['VRDEBIFCP'] + (float) $maparecupnc['VROUTRDEBIFCP'] + (float) $maparecupnc['VRESTOCREDFCP'];
        
            /* Se Total de Débitos - Total Créditos – Crédito Anterior – Deduções < 0
            Saldo Credor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções
            Saldo Devedor: 0 */
            $conta = (float)$maparecupnc['VRTOTADEBIFCP'] - (float)$maparecupnc['VRTOTACREDFCP'] - (float)$maparecupnc['VRSLDCRANTEFCP'] - (float)$maparecupnc['VRDEDUCAOFCP'];
            if($conta < 0 ){
                $maparecupnc['VRSALDOCREDFCP']   = abs($conta); // valor absoluta da conta
                $maparecupnc['VRSALDODEVEFCP']   = 0;
                $mapaapurestadonc['VRIMPRECFCP'] = 0;
                
            }
            /*Se Não
                Saldo Credor: 0
                Saldo Devedor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções */
            else{
                $maparecupnc['VRSALDOCREDFCP']   = 0; // valor absoluta da conta
                $maparecupnc['VRSALDODEVEFCP']   = abs($conta); // valor absoluta da conta
                $mapaapurestadonc['VRIMPRECFCP'] = abs($conta);
            }
            
        }
        
        // Atualiza a tabela do detalhemento por estado com o valor da soma do DIFAL + FCP;
        $mapaapurestadonc['VRIMPRECOLHER'] = (float)$mapaapurestadonc['VRIMPRECFCP'] + (float) $mapaapurestadonc['VRIMPRECDIFAL'];

        $this->saveGeral($maparecupnc,'maparecupnc');
        $this->entityManager->flush();
        
        $this->saveGeral($mapaapurestadonc,'mapaapurestadonc');
        $this->entityManager->flush();
        
        
        // Atualiza a tabela do detalhemento do mapa com o valor da somas dos impostos a recolher do DIDAL E FCP de todos os estados
        $somaValorDifalFcp = $this->factoryApuracaoDeImposto->somaValoresDifalFcp(
            array(
                "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                "NRORG" => $this->nrorg
            )
        );
        $maparecup['VRDIFALNC'] = $somaValorDifalFcp['SOMADIFAL'];
        $maparecup['VRFCPNC']   = $somaValorDifalFcp['SOMAFCP'];
        
        $this->saveGeral($maparecup,'maparecup');
        $this->entityManager->flush();
    }
    
    /*
     * Função para recontabilizar todos os valores de um mapa de apuração de
     * Impostos Gerais e seu detalhamento.
     */
    public function calculaDetalhamentoGeral($row) {
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $mapaapurac = $this->factoryApuracaoDeImposto->getMapaapurac($params);
        $maparecup  = $this->factoryApuracaoDeImposto->getMaparecup($params);
        
        $estadoInscricao  = $this->factoryApuracaoDeImposto->getEstadoInscricao(
            array(
                "CDINSCESTA" => $mapaapurac['CDINSCESTA'],
                "CDEMPRESA"  => $mapaapurac['CDEMPRESA']
            )
        );
        
        $somaDetalhamento = $this->factoryApuracaoDeImposto->getSomasDetalhamentoGeral(
            array(
                "P_SEQUMAPAPU" => $mapaapurac['NRSEQUMAPAPU']
            )    
        );
        $mapaapurac['__is_new'] = false;
        $maparecup['__is_new']  = false;
        
        $maparecup['VROUTRCRED'] = $somaDetalhamento['VROUTROSCREDITOS'];
        $maparecup['VRESTODEBI'] = $somaDetalhamento['VRESTORNODEBITO'];
        $maparecup['VROUTRDEBI'] = $somaDetalhamento['VROUTROSDEBITOS'];
        $maparecup['VRESTOCRED'] = $somaDetalhamento['VRESTORNOCREDITO'];
        $maparecup['VRDEDUCAO']  = $somaDetalhamento['VRDEDUCOES'];
        $maparecup['VRDIFALIQU'] = $somaDetalhamento['VRDIFERENCIAL'];
        $maparecup['VRDEBESPECIAL'] = $somaDetalhamento['VRDEBESPECIAL'];
        $maparecup['VRINFORMATIVO'] = $somaDetalhamento['VRINFORMATIVO'];
        
        //Total Créditos: Crédito Normal + Outros Créditos + Estorno Débitos
        $maparecup['VRTOTACRED'] = (float) $maparecup['VRNORMCRED'] + (float) $maparecup['VROUTRCRED'] + (float) $maparecup['VRESTODEBI'];
        
        //Total Débitos: Débito Normal + Outros Débitos + Estorno Crédito
        $maparecup['VRTOTADEBI'] = (float) $maparecup['VRNORMDEBI'] + (float) $maparecup['VROUTRDEBI'] + (float) $maparecup['VRESTOCRED'];
        
        /* Se Total de Débitos - Total Créditos – Crédito Anterior – Deduções < 0
        Saldo Credor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções
        Saldo Devedor: 0 */
        $conta = (float)$maparecup['VRTOTADEBI'] - (float)$maparecup['VRTOTACRED'] - (float)$maparecup['VRSLDCRANTE'] - (float)$maparecup['VRDEDUCAO'];
        if($conta < 0 ){
            $maparecup['VRSALDOCRED']  = abs($conta); // valor absoluta da conta
            $maparecup['VRSALDODEBI']  = 0;
            
            $mapaapurac['VRTOTAIMPOS'] = 0;
            
        }
        /*Se Não
            Saldo Credor: 0
            Saldo Devedor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções */
        else{
            $maparecup['VRSALDOCRED']  = 0; // valor absoluta da conta
            $maparecup['VRSALDODEBI']  = abs($conta); // valor absoluta da conta
            
            $mapaapurac['VRTOTAIMPOS'] = abs($conta);;
        }
        
        $this->saveGeral($maparecup,'maparecup');
        $this->entityManager->flush();
        
        $this->saveGeral($mapaapurac,'mapaapurac');
        $this->entityManager->flush();
        
    }
    
    /*
     * Função para recontabilizar todos os valores de um mapa de apuração de
     * Contribuições e seu detalhamento.
     */
    public function calculaDetalhamentoContribuicoes($row) {
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $mapaapurac = $this->factoryApuracaoDeImposto->getMapaapurac($params);
        // Busca a tabela de detalhamento.
        $detapuracontr = $this->factoryApuracaoDeImposto->getDetapuracontr($params);
        $somasItensDetalhamento = $this->factoryApuracaoDeImposto->getSomasItensDetalhamentoContribuicoes($params);
        $somasItensAjustes = $this->factoryApuracaoDeImposto->getSomasItensAjustesContribuicoes($params);
        $somasItensAjustesPag = $this->factoryApuracaoDeImposto->getSomasItensAjustesContribuicoesPag($params);
        
        $totalCreditos = (float) $somasItensDetalhamento["VRCREDPERANT"]
                       + (float) $detapuracontr["VRCREDITOS"]
                       + (float) $somasItensAjustes["VRAJUSTECREDENTR"]
                       - (float) $somasItensAjustes["VRAJUSTEDEBENTR"];

        $totalDebitos = (float) $detapuracontr["VRDEBITOSNC"]
                      + (float) $detapuracontr["VRDEBITOSCUM"]
                      + (float) $somasItensAjustes["VRAJUSTECREDSAIDDI"]
                      + (float) $somasItensAjustes["VRAJUSTECREDSAIDDB"]
                      - (float) $somasItensAjustes["VRAJUSTEDEBSAIDDI"]
                      - (float) $somasItensAjustes["VRAJUSTEDEBSAIDDB"];

        $totalCreditosPag = (float) $somasItensDetalhamento["VRCREDPERANT"]
                          + (float) $detapuracontr["VRCREDITOSRPAG"]
                          + (float) $somasItensAjustesPag["VRAJCREDENTRPAG"]
                          - (float) $somasItensAjustesPag["VRAJDEBENTRPAG"];

        $totalDebitosPag = (float) $detapuracontr["VRDEBITOSNCRPAG"]
                         + (float) $detapuracontr["VRDEBITOSCUMRPAG"]
                         + (float) $somasItensAjustesPag["VRAJCREDSAIDDIPAG"]
                         + (float) $somasItensAjustesPag["VRAJDEBSAIDDIPAG"]
                         - (float) $somasItensAjustesPag["VRAJCREDSAIDDBPAG"]
                         - (float) $somasItensAjustesPag["VRAJDEBSAIDDBPAG"];

        $diferencaDebitosCreditos = (float) $totalDebitos - (float) $totalCreditos;
        $saldoCredor = 0;
        // Verifica se o total de créditos é maior que o total de débitos.
        if ($diferencaDebitosCreditos < 0) {
            // Atribui o valor absoluto da diferença ao saldo credor.
            $saldoCredor = $diferencaDebitosCreditos * (-1);
            // Zera a variável para atualização de registro na tabela e
            // verificação do valor total após verificação dos valores de
            // compensação e dedução.
            $diferencaDebitosCreditos = 0;
        }

        $valorTotal = $diferencaDebitosCreditos - (float) $somasItensDetalhamento["VRCOMPENSA"] - (float) $somasItensDetalhamento["VRDEDUCAO"];
        if ($valorTotal < 0) {
            // Adiciona o valor excedente ao saldo credor.
            $saldoCredor += $valorTotal * (-1);
            // Zera a variável para atualização de registro na tabela.
            $valorTotal = 0;
        }

        $params = array(
            "NRSEQUMAPAPU"          => $row["NRSEQUMAPAPU"],
            "NRORG"                 => $this->nrorg,
            "VRCREDPERANT"          => $somasItensDetalhamento["VRCREDPERANT"],
            "VRCOMPENSA"            => $somasItensDetalhamento["VRCOMPENSA"],
            "VRDEDUCAO"             => $somasItensDetalhamento["VRDEDUCAO"],
            
            
            "VRAJUSTECREDENTR"      => $somasItensAjustes["VRAJUSTECREDENTR"],
            "VRAJUSTEDEBENTR"       => $somasItensAjustes["VRAJUSTEDEBENTR"],
            "VRAJUSTECREDSAID"      => $somasItensAjustes["VRAJUSTECREDSAIDDI"] + $somasItensAjustes["VRAJUSTECREDSAIDDB"],
            "VRAJUSTEDEBSAID"       => $somasItensAjustes["VRAJUSTEDEBSAIDDI"] + $somasItensAjustes["VRAJUSTEDEBSAIDDB"],
            "VRAJUSTECREDSAIDDI"    => $somasItensAjustes["VRAJUSTECREDSAIDDI"],
            "VRAJUSTECREDSAIDDB"    => $somasItensAjustes["VRAJUSTECREDSAIDDB"],
            "VRAJUSTEDEBSAIDDI"     => $somasItensAjustes["VRAJUSTEDEBSAIDDI"],
            "VRAJUSTEDEBSAIDDB"     => $somasItensAjustes["VRAJUSTEDEBSAIDDB"],
            
            "VRAJCREDENTRPAG"       => $somasItensAjustesPag["VRAJCREDENTRPAG"],
            "VRAJDEBENTRPAG"        => $somasItensAjustesPag["VRAJDEBENTRPAG"],
            "VRAJCREDSAIDPAG"       => $somasItensAjustesPag["VRAJCREDSAIDDIPAG"] + $somasItensAjustesPag["VRAJDEBSAIDDIPAG"],
            "VRAJDEBSAIDPAG"        => $somasItensAjustesPag["VRAJCREDSAIDDBPAG"] + $somasItensAjustesPag["VRAJDEBSAIDDBPAG"],
            "VRAJCREDSAIDDIPAG"     => $somasItensAjustesPag["VRAJCREDSAIDDIPAG"],
            "VRAJDEBSAIDDIPAG"      => $somasItensAjustesPag["VRAJDEBSAIDDIPAG"],
            "VRAJCREDSAIDDBPAG"     => $somasItensAjustesPag["VRAJCREDSAIDDBPAG"],
            "VRAJDEBSAIDDBPAG"      => $somasItensAjustesPag["VRAJDEBSAIDDBPAG"],
            
            "VRTOTCRED"             => $totalCreditos,
            "VRTOTDEB"              => $totalDebitos,
            "VRSALDOCRED"           => $saldoCredor,
            "VRTOTCONTR"            => $diferencaDebitosCreditos,
            
            "VRTOTCREDRPAG"         => $totalCreditosPag,
            "VRTOTDEBRPAG"          => $totalDebitosPag,
            "VRSALDOCREDRPAG"       => isset($saldoCredorPag) ? $saldoCredorPag : null,
            "VRSOMADEBCREDRPAG"     => isset($diferencaDebitosCreditosPag) ? $diferencaDebitosCreditosPag : null
        );
        $this->factoryApuracaoDeImposto->updateValoresDetalhamentoContribuicoes($params);

        $valorTotalGuia = $valorTotal + (float) $mapaapurac["VRJUROGUIA"] + (float) $mapaapurac["VRMULTAGUIA"];
        $params = array(
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "NRORG"         => $this->nrorg,
            "VRTOTAIMPOS"   => $valorTotal,
            "VRTOTAGUIA"    => $valorTotalGuia
        );
        $this->factoryApuracaoDeImposto->updateValorTotalMapaContribuicoes($params);
    }
    
    /*
     * Função para recontabilizar todos os valores de um mapa de apuração de
     * Imposto IRPJ/CSLL e seu detalhamento.
     */
    /*public function calculaDetalhamentoIrpjCsll($row) {
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $mapaapurac = $this->factoryApuracaoDeImposto->getMapaapurac($params);
        $maparecup  = $this->factoryApuracaoDeImposto->getMaparecup($params);
        
        $estadoInscricao  = $this->factoryApuracaoDeImposto->getEstadoInscricao(
            array(
                "CDINSCESTA" => $mapaapurac['CDINSCESTA'],
                "CDEMPRESA"  => $mapaapurac['CDEMPRESA']
            )
        );
        
        $somaDetalhamento = $this->factoryApuracaoDeImposto->getSomasDetalhamentoGeral(
            array(
                "P_SEQUMAPAPU" => $mapaapurac['NRSEQUMAPAPU']
            )    
        );
        $mapaapurac['__is_new'] = false;
        $maparecup['__is_new']  = false;
        
        $maparecup['VROUTRCRED'] = $somaDetalhamento['VROUTROSCREDITOS'];
        $maparecup['VRESTODEBI'] = $somaDetalhamento['VRESTORNODEBITO'];
        $maparecup['VROUTRDEBI'] = $somaDetalhamento['VROUTROSDEBITOS'];
        $maparecup['VRESTOCRED'] = $somaDetalhamento['VRESTORNOCREDITO'];
        $maparecup['VRDEDUCAO']  = $somaDetalhamento['VRDEDUCOES'];
        $maparecup['VRDIFALIQU'] = $somaDetalhamento['VRDIFERENCIAL'];
        $maparecup['VRDEBESPECIAL'] = $somaDetalhamento['VRDEBESPECIAL'];
        $maparecup['VRINFORMATIVO'] = $somaDetalhamento['VRINFORMATIVO'];
        
        //Total Créditos: Crédito Normal + Outros Créditos + Estorno Débitos
        $maparecup['VRTOTACRED'] = (float) $maparecup['VRNORMCRED'] + (float) $maparecup['VROUTRCRED'] + (float) $maparecup['VRESTODEBI'];
        
        //Total Débitos: Débito Normal + Outros Débitos + Estorno Crédito
        $maparecup['VRTOTADEBI'] = (float) $maparecup['VRNORMDEBI'] + (float) $maparecup['VROUTRDEBI'] + (float) $maparecup['VRESTOCRED'];
        
        /* Se Total de Débitos - Total Créditos – Crédito Anterior – Deduções < 0
        Saldo Credor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções
        Saldo Devedor: 0 *//*
        $conta = (float)$maparecup['VRTOTADEBI'] - (float)$maparecup['VRTOTACRED'] - (float)$maparecup['VRSLDCRANTE'] - (float)$maparecup['VRDEDUCAO'];
        if($conta < 0 ){
            $maparecup['VRSALDOCRED']  = abs($conta); // valor absoluta da conta
            $maparecup['VRSALDODEBI']  = 0;
            
            $mapaapurac['VRTOTAIMPOS'] = 0;
            
        }
        /*Se Não
            Saldo Credor: 0
            Saldo Devedor: Valor Absoluto  Total de Débitos - Total Créditos – Crédito Anterior – Deduções */
        /*else{
            $maparecup['VRSALDOCRED']  = 0; // valor absoluta da conta
            $maparecup['VRSALDODEBI']  = abs($conta); // valor absoluta da conta
            
            $mapaapurac['VRTOTAIMPOS'] = abs($conta);;
        }
        
        $this->saveGeral($maparecup,'maparecup');
        $this->entityManager->flush();
        
        $this->saveGeral($mapaapurac,'mapaapurac');
        $this->entityManager->flush();
        
    }
    */

    /*
     * Verifica se um mapa pode ser deletado e o exclui, removendo os registros
     * nas tabelas associadas.
     */
    public function deleteMapa($row) {
        // Verifica se há uma preparação de pagamento para o mapa.
        if ($this->existePreparacaoPagamento($row)) {
            throw new \Exception("Mapa não pode ser excluído. Foi realizada a preparação de pagamento para o imposto.");
        }
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Faz a exclusão para mapas de apuração do Simples Nacional.
            if ($this->isSimplesNacional($row)) {
                $params = array(
                    "NRORG" => $this->nrorg,
                    "CDEMPRESA" => $row["CDEMPRESA"],
                );
                // Busca último mapa para a empresa selecionada.
                $ultimoMapa = $this->factoryApuracaoDeImposto->buscaUltimoMapaSimples($params);

                // Verifica se o último mapa corresponde ao mapa selecionado.
                if ($ultimoMapa['NRSEQUMAPAPU'] == $row['NRSEQUMAPAPU']) {
                    $params = array(
                        "NRORG" => $this->nrorg,
                        "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    );

                    // Exclui os dados das tabelas relacionadas.
                    $this->factoryApuracaoDeImposto->atualizaNotasFiscaisMapaApuracao($params);
                    $this->entityManager->flush();

                    $this->factoryApuracaoDeImposto->deletaItdetalheapursn($params);
                    $this->entityManager->flush();

                    $this->factoryApuracaoDeImposto->deletaDetalheapursn($params);
                    $this->entityManager->flush();

                    $this->factoryApuracaoDeImposto->deletaMapaapuracsn($params);
                    $this->entityManager->flush();

                    $connection->commit();
                }
                else {
                    throw new \Exception("Apenas o último mapa pode ser excluído.");
                }
            }
            else {
                $params = array(
                    "NRORG"         => $this->nrorg,
                    "CDEMPRESA"     => $row["CDEMPRESA"],
                    "CDIMPOSTO"     => $row["CDIMPOSTO"],
                    "CDINSCESTA"    => $row['CDINSCESTA'],
                    "CDINSCMUNI"    => $row['CDINSCMUNI'],
                    "IDTIPOAPURA"   => $row['IDTIPOAPURA']
                );
                // Busca último mapa para a empresa selecionada.
                $ultimoMapa = $this->factoryApuracaoDeImposto->buscaUltimoMapa($params);
    
                // Verifica se o último mapa corresponde ao mapa selecionado.
                if ($ultimoMapa['NRSEQUMAPAPU'] == $row['NRSEQUMAPAPU']) {
                    
                    $params = array(
                        "NRORG" => $this->nrorg,
                        "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    );
                    
                    if( $this->isContribuicoes($row) ){
                        $paramFiscal = $this->factoryApuracaoDeImposto->getParamfiscal( array('NRORG' =>$this->nrorg ) );
                        
                        $this->factoryApuracaoDeImposto->deletaGuiacompapucontr($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaItdetalheapur($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaDetalheapur($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaAjusteapurcontr($params);
                        $this->entityManager->flush();
                    
                        $this->factoryApuracaoDeImposto->atualizaNotasFiscaisGerais(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'DTREFMAPAPU'  =>$paramFiscal['IDNFFORAPERAPU']
                            )
                        );
                        $this->entityManager->flush();
                       
                        $this->factoryApuracaoDeImposto->atualizaImpbaixarec(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'DTREFMAPAPU'  =>$paramFiscal['IDNFFORAPERAPU']
                            )
                        );
                        $this->entityManager->flush();

                        $this->factoryApuracaoDeImposto->atualizaNotasFiscaisCompensacoes(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'DTREFMAPAPU'  =>$paramFiscal['IDNFFORAPERAPU']
                            )
                        );
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->atualizaItemvenoutimpos(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'DTREFMAPAPU'  =>$paramFiscal['IDNFFORAPERAPU']
                            )
                        );
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaErronfprevalid($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaNfprevalidfis($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaMapaapurac($params);
                        $this->entityManager->flush();
                    
                    }
                    else if( $this->isImpostosIrpjCsll($row) ){
                        
                        $this->factoryApuracaoDeImposto->deletaItdetalheapur($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaItdetapubalctirpj($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaBalctredsuspirpj($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaRecebrutaliqmapa($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaMapaapuimpofed($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaLucropresumido(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'CDIMPOSTO'    =>$row['CDIMPOSTO'],
                                   'CDEMPRESA'    =>$row['CDEMPRESA']
                            )
                        );
                        $this->entityManager->flush();
                    
                        $this->factoryApuracaoDeImposto->deletaGuiacomplmapa($params);
                        $this->entityManager->flush();
                     
                        $this->factoryApuracaoDeImposto->deletaMapaapurac($params);
                        $this->entityManager->flush();
                    
                    }else{
                        $this->factoryApuracaoDeImposto->deleteInfadicmapa($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->proceduraDeletarMapaGeral($params);
                        $this->entityManager->flush();
                        
                        
                        $parcelasAtivo = $this->factoryApuracaoDeImposto->buscaParcelasAtivo(
                            array( 'P_DTFINAL'   =>$row['DTFIMPERIODO'],
                                   'P_DTINICIAL' =>$row['DTINIPERIODO'],
                                   'P_INSCRICAO' => array_key_exists ('CDINSCESTA',$row) ? $row['CDINSCESTA'] : null,
                                   'P_EMPRESA'   =>$row['CDEMPRESA']
                            )
                        );
                        if(count ($parcelasAtivo) > 0 ){
                            foreach ($parcelasAtivo as $valor) {
                                if( $valor['NRSEQUMAPAPU'] === $row['NRSEQUMAPAPU'] ){
                                    $this->factoryApuracaoDeImposto->updateParclancativ(
                                        array( 'NRPARCATIV'  =>$valor['NRPARCATIV'],
                                               'NRSEQITPROD' =>$valor['NRSEQITPROD'],
                                               'NRLANCTONF'  =>$valor['NRLANCTONF'],
                                               'CDFILIAL'    =>$valor['CDFILIAL']
                                        )
                                    );
                                    $this->entityManager->flush();
                                    
                                }
                            }
                        }
                        
                        $this->factoryApuracaoDeImposto->deletaLucropresumido(
                            array( 'NRORG'        =>$this->nrorg,
                                   'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                                   'CDIMPOSTO'    =>$row['CDIMPOSTO'],
                                   'CDEMPRESA'    =>$row['CDEMPRESA']
                            )
                        );
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaMapaapuimpofed($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaGuiacomplmapa($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaAjustmapaapursg($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaDetaufmapaest($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaMapaapurestado($params);
                        $this->entityManager->flush();
                        
                        $this->factoryApuracaoDeImposto->deletaMapaapurac($params);
                        $this->entityManager->flush();
                    }
                    $connection->commit();
                }
                else {
                    throw new \Exception("Apenas o último mapa pode ser excluído.");
                }
            }

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /*
     * Cria registro na tabela de detalhamento para mapas de apuração de
     * Contribuições, se este já não existir.
     */
    public function insertDetalhamento($row) {
        $params = array(
            "NRORG" => $this->nrorg,
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
        );
        if ($this->isContribuicoes($row)) {
            // Cria o registro na tabela de detalhamento de Contribuições.
            $this->factoryApuracaoDeImposto->insertDetapuracontr($params);
        }else if( !$this->isSimplesNacional($row) ){
            if($this->isImpostosGerais($row)){
                // Cria o registro na tabela de detalhamento de Impostos Gerais.
                $this->factoryApuracaoDeImposto->insertMaparecup($params);
            }    
        }
    }
    
    public function salvarCreditoAnteriorDifalFcp($row) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $this->factoryApuracaoDeImposto->salvarCreditoAnteriorDifalFcp(
                array(
                    "NRORG"            => $this->nrorg,
                    "NRSEQMAPARECUPNC" => $row["NRSEQMAPARECUPNC"],
                    "VRSLDCRANTEDIFAL" => $row["VRSLDCRANTEDIFAL"],
                    "VRSLDCRANTEFCP"   => $row["VRSLDCRANTEFCP"],
                )
            );
            
            $this->entityManager->flush();
            $this->calculaDetalhamentoDifalFcp($row); 
            
            $connection->commit();
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function updateDetalhamento($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            if ($this->isContribuicoes($row)) {
                /*
                Será necessário incluir/updatar uma linha na tabela ITDETAPUCONTR 
                para manter a compatibilidade com o Delphi). 
                */
               
                //Apaga a linha se a mesma já existir, para evitar valores incorretos:
                $this->factoryApuracaoDeImposto->deleteItdetapucontr(array("P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]) );
                $this->entityManager->flush();
                
                $sequencial = $this->factoryApuracaoDeImposto->buscaSequencialItdetapucontr(array("P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]) );
                $params = array( 
                    "P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"], 
                    "V_SEQITCREANT"  => $sequencial["NRSEQITDETAPU"], 
                    "V_VRCREMAPANT"  => $row["VRCREDPERANT"],
                    "P_CDCONTRATEIO" => isset($row["CDCONTRATEIO"])? $row["CDCONTRATEIO"]: null,
                    "P_CDEMPRESA"    => isset($row["CDEMPRESA"]   )? $row["CDEMPRESA"]   : null,
                    "P_CDLOTE"       => isset($row["CDLOTE"]      )? $row["CDLOTE"]      : null
                );
                $this->factoryApuracaoDeImposto->insertItdetapucontr($params);
                $this->entityManager->flush();
                
                $this->calculaDetalhamentoContribuicoes($row);
                $this->entityManager->flush();
            }
            else if ($this->isImpostosGerais($row)){
                if($row['CDIMPOSTO'] === self::IMPOSTO_ICMSST){
                    
                    $this->factoryApuracaoDeImposto->updateCreditoAnterirorIcmsSt(
                        array(
                            "VRCREDANTERIOR"   => $row["VRCREDANTERIOR"],
                            "NRORG"            => $row["NRORG"],
                            "NRSEQUMAPAPU"     => $row["NRSEQUMAPAPU"],
                            "NRSEQMAPAAPUREST" => $row["NRSEQMAPAAPUREST"],
                            "CDPAIS"           => $row["CDPAIS"],
                            "SGESTADO"         => $row["SGESTADO"]
                        )
                    );
                    $this->entityManager->flush();
                    
                    $this->calculaDetalhamentoIcmsSt($row); 
                }
                else{
                    $params = array(
                    "NRORG" => $this->nrorg,
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    "VRSLDCRANTE" => $row["VRSLDCRANTE"]
        
                    );
                    // Atualiza o registro na tabela de detalhamento de ImpostosGerais.
                    $this->factoryApuracaoDeImposto->updateMaparecup($params);
                    $this->entityManager->flush();
                    
                    $this->calculaDetalhamentoGeral($row);
                    $this->entityManager->flush();    
                }
                
                
            }
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
            
    }
   
    public function salvarArrecadacaoIcmsSt($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            
            if($row['__is_new']){
                $row['NRSEQUDETUF'] = str_pad( $this->factoryApuracaoDeImposto->getcodigoDetaufmapaest(
                    array(
                        "P_NRSEQUMAPAPU"   => $row["NRSEQUMAPAPU"],
                        "P_CDPAIS"         => $row["CDPAIS"],
                        "P_SGESTADO"       => $row["SGESTADO"],
                        "NRSEQMAPAAPUREST" => $row["NRSEQMAPAAPUREST"]
                    )
                )['NRSEQUDETUF'], 10, "0", STR_PAD_LEFT);
            }
            $this->saveGeral($row,'detaufmapaest');
            $this->entityManager->flush();
            
            $connection->commit();
        }
         catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
            
    }
    
    public function salvarDetalhamentoInss($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
           
            $dataSet = new DataSet('mapaapurac', [$row]);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
            
            $this->calculaDetalhamentoInss($row); 
            $this->entityManager->flush();    
            
                
            $connection->commit();
        }
         catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
            
    }

    /*
     * Salva os dados alterados na tela de arrecadação para mapas de apuração
     * de Contribuições.
     */
    public function saveArrecadacaoContribuicoes($row) {
        // Contabiliza valor total da guia.
        $row["VRTOTAGUIA"] = (float) $row["VRTOTAIMPOS"] + (float) $row["VRJUROGUIA"] + (float) $row["VRMULTAGUIA"];

        // Monta os dados para atualizar no banco de dados.
        $params = array(
            "NRORG"         => $this->nrorg,
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "DTVENCGUIA"    => $row["DTVENCGUIA"],
            "CDRECENCUMU"   => $row["CDRECENCUMU"],
            "CDRECECUMU"    => $row["CDRECECUMU"],
            "CDVARIACAO"    => $row["CDVARIACAO"],
            "IDPERIODICREC" => $row["IDPERIODICREC"],
            "IDDEBSCPINC"   => $row["IDDEBSCPINC"],
            "DSHISTGUIA"    => $row["DSHISTGUIA"],
            "VRTOTAIMPOS"   => $row["VRTOTAIMPOS"],
            "VRJUROGUIA"    => $row["VRJUROGUIA"],
            "VRMULTAGUIA"   => $row["VRMULTAGUIA"],
            "VRTOTAGUIA"    => $row["VRTOTAGUIA"],
        );
        $this->factoryApuracaoDeImposto->saveArrecadacaoContribuicoes($params);
    }

    public function salvarArrecadacaoGeral($row) {
        $row["VRTOTAIMPOS"] = $row["VRTOTAIMPOS"] ? $row["VRTOTAIMPOS"] = $row["VRTOTAIMPOS"] : $row["VRTOTAIMPOS"] = 0;
        $row["VRJUROGUIA"]  = $row["VRJUROGUIA"]  ? $row["VRJUROGUIA"]  = $row["VRJUROGUIA"]  : $row["VRJUROGUIA"]  = 0;
        $row["VRMULTAGUIA"] = $row["VRMULTAGUIA"] ? $row["VRMULTAGUIA"] = $row["VRMULTAGUIA"] : $row["VRMULTAGUIA"] = 0;
        
        // Contabiliza valor total da guia.
        $row["VRTOTAGUIA"] = (float) $row["VRTOTAIMPOS"] + (float) $row["VRJUROGUIA"] + (float) $row["VRMULTAGUIA"];
        // Monta os dados para atualizar no banco de dados.
        
        $params = array(
            "NRORG"        => $this->nrorg,
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "DTVENCGUIA"   => $row["DTVENCGUIA"],
            "DSHISTGUIA"   => $row["DSHISTGUIA"],
            "VRTOTAIMPOS"  => $row["VRTOTAIMPOS"],
            "VRJUROGUIA"   => $row["VRJUROGUIA"],
            "VRMULTAGUIA"  => $row["VRMULTAGUIA"],
            "VRTOTAGUIA"   => $row["VRTOTAGUIA"],
            "CDRECEGUIA"   => $row["CDRECEGUIA"]
        );
        $this->factoryApuracaoDeImposto->salvarArrecadacaoGeral($params);
    }

    public function salvarGuiasComplementares ($row){
        //$connection = $this->entityManager->getConnection();
        //$connection->beginTransaction();
        if($row['__is_new']){
            if($row['DATASOURCENAME'] == 'guiacomplmapa'){
                $params = array(
                    "NRORG"        => $this->nrorg,
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
                );
                
                $row['NRSEQGUIACOMPL']  = str_pad( $this->factoryApuracaoDeImposto->maiorCodigoGuiasComplementares($params)['NRSEQGUIACOMPL']+1 , 10, "0", STR_PAD_LEFT);
                $row['VRJUROSGUIACOMP'] = '';
                $row['VRMULTAGUIACOMP'] = '';
                $row['VRTOTALGUIACOMP'] = '';
            } else if($row['DATASOURCENAME'] == 'guiacompapucontr'){
                $params = array(
                    "NRORG"        => $this->nrorg,
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
                );
                
                $row['NRSEQGUIACOMPL']  = str_pad( $this->factoryApuracaoDeImposto->maiorCodigoGuiasComplementaresContrib($params)['NRSEQGUIACOMPL']+1 , 10, "0", STR_PAD_LEFT);
                $row['VRJUROSGUIACOMP'] = '';
                $row['VRMULTAGUIACOMP'] = '';
                $row['VRTOTALGUIACOMP'] = '';
            } 
        }
        $this->setCamposAuditoria($row);
        $dataSet = new DataSet($row['DATASOURCENAME'], [$row]);
        $this->dataSourceManager->persist($dataSet);
        //$this->entityManager->flush();
        //$connection->commit();
    }
    
    /*
        Função que recebe uma row e cria um  DataSet e 
        realiza o procedimento de salvar
    */
    public function saveGeral($row, $dataSourceName){
        $this->setCamposAuditoria($row);
        
        $dataSet = new DataSet($dataSourceName, [$row]);
        $this->dataSourceManager->persist($dataSet);
        
        $this->entityManager->flush();
    }
    
    /*
        Funcão que salvar o sub detalhemto do INSS
    */
    public function salvarSubDetalhamentoInss ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->saveGeral($row,'itdetapucontr'); 
            $this->entityManager->flush();
            
            $this->calculaDetalhamentoInss($row); 
            $this->entityManager->flush();
     
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    /*
        Funcão que salvar o sub detalhemto por Estado
        do ICMS ST
    */
    public function salvarSubDetalhamentoIcmsSt ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $ajustmapaapursgAntigo = $this->factoryApuracaoDeImposto->getAjustmapaapursg(
                array(
                    "NRSEQUMAPAPU"     => $row['NRSEQUMAPAPU'],
                    "CDPAIS"           => $row['CDPAIS'],
                    "SGESTADO"         => $row['SGESTADO'],
                    "NRSEQAJUSAPURSG"  => $row['NRSEQAJUSAPURSG'],
                    "NRORG"            => $row['NRORG'],
                    "NRSEQMAPAAPUREST" => $row['NRSEQMAPAAPUREST']
                )
            );    
            if($ajustmapaapursgAntigo){
                $ajustmapaapursgAntigo['NRSEQITMAPR']  = $ajustmapaapursgAntigo['NRSEQAJUSAPURSG'];
                $ajustmapaapursgAntigo['IDTIPOITMAPR'] = $ajustmapaapursgAntigo['IDTPAJUSTEAPSG'] ==='D' ?'OD':'OC';    
            }
            
            
            $this->verificaItensComposicoes($row, $ajustmapaapursgAntigo);
            
            $this->saveGeral($row,'ajustmapaapursg'); 

            $this->entityManager->flush();
            
            //Valores default
            $row['NRSEQITMAPR']    = $row['NRSEQAJUSAPURSG'];
            $row['IDTIPOITMAPR']   = $row['IDTPAJUSTEAPSG'] === 'D' ? 'OD' : 'OC';
            $row['VRITEMMAPR']     = $row['VRAJUSTEAPURSG'];
            $row['DSITEMMAPR']     = $row['DSAJUSTEAPURSG'];
            $row['CDAJUSTEITMAPA'] = $row['CDAJUSTEAPURSG'];
            $this->insereValoresItensComposicoes($row);
            
            // Atualiza todos os intens de composição para o substituto solidario
            // correspondente da tabela AJUSTMAPAAPURSG
            $this->factoryApuracaoDeImposto->updateSubstitutoSolidarioIcmsSt(
                array(
                    "CDFILIAL"      => $row['CDFILIAL'],
                    "IDTIPOITMAPR"  => $row['IDTPAJUSTEAPSG'] === 'D'?'OD':'OC',
                    "NRSEQUMAPAPU"  => $row['NRSEQUMAPAPU'],
                    "NRLANCTONF"    => $row['NRLANCTONF'],
                    "NRSEQITMAPR"   => $row['NRSEQAJUSAPURSG'],
                    "IDSUBSTSOLIDST"=> $row['IDSUBSTSOLIDST']
                )
            ); 
            $this->entityManager->flush();
            
            
            $this->calculaDetalhamentoIcmsSt($row); 

            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    /*
        Função que salva as informações do sub detalahemto
        de impostos gerais que são abertas nos icones 
        presentes na aba de detalhamento em apuração de Impostos 
        Gerais.
    */
    public function salvarSubDetalhamentoImpostosGerais ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $itmaparecupAntigo = $this->factoryApuracaoDeImposto->getItmaparecup(
                array(
                    "NRSEQUMAPAPU" => $row['NRSEQUMAPAPU'],
                    "IDTIPOITMAPR" => $row['IDTIPOITMAPR'],
                    "NRSEQITMAPR"  => $row['NRSEQITMAPR']
                ) 
            );        
            
            $this->verificaItensComposicoes($row, $itmaparecupAntigo);
            
            $this->saveGeral($row,'itmaparecup'); 
            $this->entityManager->flush();
            
            $this->insereValoresItensComposicoes($row);
            
            $this->calculaDetalhamentoGeral($row); 
            $this->entityManager->flush();
     
        
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    /* Função que realizará o procedimento de save de todas as abas
    que serão abertas apartir dos botões existentes na aba de detalhamento
    da apuração do ICMS (DIFAL/FCP */
    public function salvarSubDetalhamentoIcmsDifalFcp ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            if($row['__is_new'])
                $row['NRSEQITMAPRECNC'] = $this->newCode->getCodeByTableName('ITMAPARECUPNC', $this->nrorg);
                      
            $this->saveGeral($row,'itmaparecupnc'); 
            $this->entityManager->flush();
            
            $this->calculaDetalhamentoDifalFcp($row); 
            
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function deletarSubDetalhamentoImpostosGerais ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            if($row['CDIMPOSTO']===self::IMPOSTOS_INSS){
                $dataSet = new DataSet('itdetapucontr', [$row]);
                $this->dataSourceManager->delete($dataSet);
                $this->entityManager->flush();
                
                $this->calculaDetalhamentoInss($row); 
                $this->entityManager->flush();    
            }
            else if($row['CDIMPOSTO']===self::IMPOSTO_ICMSST){
                
                $this->factoryApuracaoDeImposto->deleteTodosItensComposicao(
                    array(
                        "CDFILIAL"      => $row['CDFILIAL'],
                        "IDTIPOITMAPR"  => $row['IDTPAJUSTEAPSG']==='D'?'OD':'OC',
                        "NRSEQUMAPAPU"  => $row['NRSEQUMAPAPU'],
                        "NRLANCTONF"    => $row['NRLANCTONF'],
                        "NRSEQITMAPR"   => $row['NRSEQAJUSAPURSG'],
                    )
                ); 
                $this->entityManager->flush();
                
                $dataSet = new DataSet('ajustmapaapursg', [$row]);
                $this->dataSourceManager->delete($dataSet);
                $this->entityManager->flush();
                
                $this->calculaDetalhamentoIcmsSt($row); 
                $this->entityManager->flush();
            }
            else{
                $this->factoryApuracaoDeImposto->deleteTodosItensComposicao(
                    array(
                        "CDFILIAL"      => $row['CDFILIAL'],
                        "IDTIPOITMAPR"  => $row['IDTIPOITMAPR'],
                        "NRSEQUMAPAPU"  => $row['NRSEQUMAPAPU'],
                        "NRLANCTONF"    => $row['NRLANCTONF'],
                        "NRSEQITMAPR"   => $row['NRSEQITMAPR'],
                    )
                ); 
                $this->entityManager->flush();
                
                $dataSet = new DataSet('itmaparecup', [$row]);
                $this->dataSourceManager->delete($dataSet);
                $this->entityManager->flush();
                
                $this->calculaDetalhamentoGeral($row); 
                $this->entityManager->flush();    
            }
            
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function deletarSubDetalhamentoDifalFcp ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $dataSet = new DataSet('itmaparecupnc', [$row]);
            $this->dataSourceManager->delete($dataSet);
            $this->entityManager->flush();
            
            $this->calculaDetalhamentoDifalFcp($row); 

            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function salvarItensComposicoes ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $mapaapurac = $this->factoryApuracaoDeImposto->getMapaapurac(
                array(
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    "NRORG"        => $this->nrorg
                )
            );
            
            $item = $this->factoryApuracaoDeImposto->getItemComposicao(
                array(  'P_NRSEQUMAPAPU'     => $row['NRSEQUMAPAPU'],
                        'P_IDTIPOITMAPR'     => $row['IDTIPOITMAPR'],
                        'P_NRSEQITMAPR'      => $row['NRSEQITMAPR'],
                        'P_CDFILIAL'         => $row['CDFILIAL'],
                        'P_NRLANCTONF'       => $row['NRLANCTONF'],
                        'P_NRSEQITPROD'      => $row['NRSEQITPROD'],
                        'P_CDIMPOSTO'        => $row['CDIMPOSTO'],
                        'P_NRITPROIMPOS'     => $row['NRITPROIMPOS'],
                )
            );
            
            if($row['IDCOMPOE']==='S'){
                if(empty($item)){
                    $this->factoryApuracaoDeImposto->insertDetitmaparecup(
                        array(  'P_NRSEQUMAPAPU'     => $row['NRSEQUMAPAPU'],
                                'P_IDTIPOITMAPR'     => $row['IDTIPOITMAPR'],
                                'P_NRSEQITMAPR'      => $row['NRSEQITMAPR'],
                                'P_CDFILIAL'         => $row['CDFILIAL'],
                                'P_NRLANCTONF'       => $row['NRLANCTONF'],
                                'P_NRSEQITPROD'      => $row['NRSEQITPROD'],
                                'P_CDIMPOSTO'        => $row['CDIMPOSTO'],
                                'P_NRITPROIMPOS'     => $row['NRITPROIMPOS'],
                                'P_NRSEQDETITMAPA'   => $this->newCode->getCodeByTableName('DETITMAPARECUP', $this->nrorg),
                                'P_NRORG'            => $this->nrorg,
                        )
                    );
                    $this->entityManager->flush();
                }
            }else{
                $this->factoryApuracaoDeImposto->deleteDetitmaparecup(
                    array(  'NRSEQDETITMAPA' => $row['NRSEQDETITMAPA'],
                            'NRORG'          => $this->nrorg,
                    )
                );
                $this->entityManager->flush();
                
            }
            
            // Procedimento do imposto ST para alterar valor do imposto e 
            // base de calculo qunado o IMPOSTO por substituto solidario
            if( $mapaapurac['CDIMPOSTO'] ===self::IMPOSTO_ICMSST ){
                if($row['IDSUBSTSOLIDST']==='S' && $row['IDCOMPOE']==='S' )
                    $this->factoryApuracaoDeImposto->updateValorSubstitutoSolidarioIcmsSt(
                        array(  'VRIMPOSTOIT'     => $row['VRIMPOSTO'],
                                'IDSUBSTSOLIDS'   => $row['IDSUBSTSOLIDST'],
                                'NRSEQDETITMAPA'  => $row['NRSEQDETITMAPA'],
                                'NRORG'           => $row['NRORG']
                        )
                    );
            }
            
            $soma = $this->factoryApuracaoDeImposto->getSomaItensComposicaoNotasMarcadas(
                array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                        'P_IDTIPOITMAPR' => $row['IDTIPOITMAPR'],
                        'P_NRSEQUMAPAPU' => $row['NRSEQUMAPAPU'],
                        'P_NRLANCTONF'   => $row['NRLANCTONF'],
                        'P_NRSEQITMAPR'  => $row['NRSEQITMAPR']
                )
            );
            $this->factoryApuracaoDeImposto->updateSomaItensComposicao(
                array(  "NRSEQITMAPR"    => $row['NRSEQITMAPR'],
                        'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                        'IDTIPOITMAPR'   => $row['IDTIPOITMAPR'],
                        'NRORG'          => $this->nrorg,
                        'VALOR'          => $soma['VALOR'] 
                )
            );
            $this->entityManager->flush();
            
            //Procedimento para apuração do ICMS ST
            if( $mapaapurac['CDIMPOSTO'] ===self::IMPOSTO_ICMSST ){
                $this->factoryApuracaoDeImposto->updateSomaAjustmapaapursg(
                    array(  "NRSEQAJUSAPURSG" => $row['NRSEQITMAPR'],
                            'NRSEQUMAPAPU'    => $row['NRSEQUMAPAPU'],
                            'IDTPAJUSTEAPSG'  => $row['IDTIPOITMAPR'] = $row['IDTIPOITMAPR'] ==='OD' ? 'D' :'C',
                            'NRORG'           => $this->nrorg,
                            'VALOR'           => $soma['VALOR']
                    )
                );
                $this->entityManager->flush();
                
                $this->calculaDetalhamentoIcmsSt($row); 
                $this->entityManager->flush();
            }
            //Outros Impostos Gerais
            else{
                $this->calculaDetalhamentoGeral($row); 
                $this->entityManager->flush();
            }
            
            $connection->commit();
            return $soma;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    /*
        Soma de todos os valores dos itens da 
        nota, marcados ou não.
    */
    public function controlaValorItensComposicao ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
        
            $soma = $this->factoryApuracaoDeImposto->getSomaItensComposicaoTodasNotas(
                array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                        'P_CDIMPOSTO'    => $row['CDIMPOSTO'],
                        'P_NRLANCTONF'   => $row['NRLANCTONF']
                )
            );
            $connection->commit();
            return $soma;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    /* Caso o campo compor C197 ou nota seja alterado,
    faz o procedimento de delete em tabelas relacionadas.*/
    public function verificaItensComposicoes ($row, $itmaparecupAntigo){
        if($itmaparecupAntigo){
            if( $itmaparecupAntigo['IDCOMPOEC197']!=$row['IDCOMPOEC197']  ||  $itmaparecupAntigo['NRLANCTONF']!=$row['NRLANCTONF']){
                $this->factoryApuracaoDeImposto->deleteItensComposicaoNotaFiscal(
                    array(
                        "NRSEQUMAPAPU" => $itmaparecupAntigo['NRSEQUMAPAPU'],
                        "NRSEQITMAPR"  => $itmaparecupAntigo['NRSEQITMAPR'],
                        "CDFILIAL"     => $itmaparecupAntigo['CDFILIAL'],
                        "NRLANCTONF"   => $itmaparecupAntigo['NRLANCTONF'],
                        "IDTIPOITMAPR" => $itmaparecupAntigo['IDTIPOITMAPR'],
                        "NRORG"        => $this->nrorg
                    ) 
                );
                $this->entityManager->flush();
            }
                
        }
    }
    
    /*
        Insere os valores dos itens da notas
        para compor o c197
    */
    public function insereValoresItensComposicoes ($row){
        try {
            $soma = $this->factoryApuracaoDeImposto->getSomaItensComposicaoNotasMarcadas(
                array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                        'P_IDTIPOITMAPR' => $row['IDTIPOITMAPR'],
                        'P_NRSEQUMAPAPU' => $row['NRSEQUMAPAPU'],
                        'P_NRLANCTONF'   => $row['NRLANCTONF'],
                        'P_NRSEQITMAPR'  => $row['NRSEQITMAPR']
                )
            );
            
            $mapaapurac = $this->factoryApuracaoDeImposto->getMapaapurac(
                array(
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    "NRORG"        => $this->nrorg
                )
            );
            
            //caso a soma das notas marcadas for dirente da soma da row, insere todas os itens para comporo C197
            if(isset($soma['VALOR'])){    
                if( ($row['IDCOMPOEC197'] ==='I') && ($soma['VALOR'] != floatval($row['VRITEMMAPR'])) ){
                    $itens = $this->factoryApuracaoDeImposto->getItensComposicaoNotaFiscal(
                        array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                                'P_IDTIPOITMAPR' => $row['IDTIPOITMAPR'],
                                'P_NRSEQUMAPAPU' => $row['NRSEQUMAPAPU'],
                                'P_NRLANCTONF'   => $row['NRLANCTONF'],
                                'P_CDIMPOSTO'    => $mapaapurac['CDIMPOSTO'],
                                'P_NRSEQITMAPR'  => $row['NRSEQITMAPR']
                        )
                    );
        
                    // Deleta itens para inserir todos novamente
                    $this->factoryApuracaoDeImposto->deleteItensComposicaoNotaFiscal(
                        array(
                            "NRSEQUMAPAPU" => $row['NRSEQUMAPAPU'],
                            "NRSEQITMAPR"  => $row['NRSEQITMAPR'],
                            "CDFILIAL"     => $row['CDFILIAL'],
                            "NRLANCTONF"   => $row['NRLANCTONF'],
                            "IDTIPOITMAPR" => $row['IDTIPOITMAPR'],
                            "NRORG"        => $row['NRORG'],
                        ) 
                    );
                    $this->entityManager->flush();
                    /*
                        Procedimento necessário para fazer que a tela funcione tanto 
                        no delphi quanto mobile. Manipulando registros na tabela ITMAPARECUP 
                    */
                    if($mapaapurac['CDIMPOSTO']=== self:: IMPOSTO_ICMSST ){
                        $itmaparecupAntigo = $this->factoryApuracaoDeImposto->getItmaparecup(
                            array(
                                "NRSEQUMAPAPU" => $row['NRSEQUMAPAPU'],
                                "IDTIPOITMAPR" => $row['IDTIPOITMAPR'],
                                "NRSEQITMAPR"  => $row['NRSEQITMAPR']
                            ) 
                        );  
                        if($itmaparecupAntigo){
                            $this->factoryApuracaoDeImposto->updateItmaparecup(
                                array(  'NRORG'          => $row['NRORG'],
                                        'NRSEQITMAPR'    => $row['NRSEQITMAPR'],
                                        'DSITEMMAPR'     => $row['DSITEMMAPR'],
                                        'VRITEMMAPR'     => $this->factoryApuracaoDeImposto->getSomaItensComposicaoTodasNotas(
                                                                array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                                                                        'P_CDIMPOSTO'    => $row['CDIMPOSTO'],
                                                                        'P_NRLANCTONF'   => $row['NRLANCTONF']
                                                                )
                                                            )['VRIMPOSTO'],
                                        'VROUTROSITMAPR' => $row['VROUTROSITMAPR'],
                                        'NRLANCTONF'     => $row['NRLANCTONF'],
                                        'CDFILIAL'       => $row['CDFILIAL'],
                                        'CDAJUSTEITMAPA' => $row['CDAJUSTEITMAPA'],
                                        'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                                        'IDTIPOITMAPR'   => $row['IDTIPOITMAPR'],
                                        'IDCOMPOEC197'   => $row['IDCOMPOEC197'],
                                        'CDCONTRATEIO'   => $row['CDCONTRATEIO']
                                        
                                )
                            );
                            $this->entityManager->flush();
                            
                        }
                        else{
                            $this->factoryApuracaoDeImposto->insertItmaparecup(
                                array(  'NRORG'          => $row['NRORG'],
                                        'NRSEQITMAPR'    => $row['NRSEQITMAPR'],
                                        'DSITEMMAPR'     => $row['DSITEMMAPR'],
                                        'VRITEMMAPR'     => $this->factoryApuracaoDeImposto->getSomaItensComposicaoTodasNotas(
                                                                array(  'P_CDFILIAL'     => $row['CDFILIAL'],
                                                                        'P_CDIMPOSTO'    => $row['CDIMPOSTO'],
                                                                        'P_NRLANCTONF'   => $row['NRLANCTONF']
                                                                )
                                                            )['VRIMPOSTO'],
                                        'VROUTROSITMAPR' => $row['VROUTROSITMAPR'],
                                        'NRLANCTONF'     => $row['NRLANCTONF'],
                                        'CDFILIAL'       => $row['CDFILIAL'],
                                        'CDAJUSTEITMAPA' => $row['CDAJUSTEITMAPA'],
                                        'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                                        'IDTIPOITMAPR'   => $row['IDTIPOITMAPR'],
                                        'IDCOMPOEC197'   => $row['IDCOMPOEC197'],
                                        'CDCONTRATEIO'   => $row['CDCONTRATEIO']
                                        
                                )
                            );
                            $this->entityManager->flush();
                        }
                    }
                    
                    // Insere todos os Itens na tabela
                    foreach ($itens as $item) {
                        $this->factoryApuracaoDeImposto->insertDetitmaparecup(
                            array(  'P_NRSEQUMAPAPU'     => $item['NRSEQUMAPAPU'],
                                    'P_IDTIPOITMAPR'     => $item['IDTIPOITMAPR'],
                                    'P_NRSEQITMAPR'      => $item['NRSEQITMAPR'],
                                    'P_CDFILIAL'         => $item['CDFILIAL'],
                                    'P_NRLANCTONF'       => $item['NRLANCTONF'],
                                    'P_NRSEQITPROD'      => $item['NRSEQITPROD'],
                                    'P_CDIMPOSTO'        => $item['CDIMPOSTO'],
                                    'P_NRITPROIMPOS'     => $item['NRITPROIMPOS'],
                                    'P_NRSEQDETITMAPA'   => $this->newCode->getCodeByTableName('DETITMAPARECUP', $this->nrorg),
                                    'P_NRORG'            => $this->nrorg,
                            )
                        );
                        $this->entityManager->flush();
                    }
                }
                else{
                    //Caso seja ICMS ST, atualizada o valor da tabela ITMAPARECUP
                    if($mapaapurac['CDIMPOSTO'] === self:: IMPOSTO_ICMSST ){
                        $this->factoryApuracaoDeImposto->updateItmaparecup(
                                array(  'NRORG'          => $row['NRORG'],
                                        'NRSEQITMAPR'    => $row['NRSEQITMAPR'],
                                        'DSITEMMAPR'     => $row['DSITEMMAPR'],
                                        'VRITEMMAPR'     => $row['VRAJUSTEAPURSG'],
                                        'NRLANCTONF'     => $row['NRLANCTONF'],
                                        'VROUTROSITMAPR' => $row['VROUTROSITMAPR'],
                                        'CDFILIAL'       => $row['CDFILIAL'],
                                        'CDAJUSTEITMAPA' => $row['CDAJUSTEITMAPA'],
                                        'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                                        'IDTIPOITMAPR'   => $row['IDTIPOITMAPR'],
                                        'IDCOMPOEC197'   => $row['IDCOMPOEC197']
                                        
                                )
                            );
                            $this->entityManager->flush();
                    }
                }
            }
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function salvarInformacoesAdicionais($row){
        try{
            if($row['__is_new'])
                $row['NRSEQINFADIC'] = $this->factoryApuracaoDeImposto->getCodigoinfadicmapa( array('NRSEQUMAPAPU'=> $row['NRSEQUMAPAPU'], 'NRORG' => $this->nrorg) )['NRSEQINFADIC'];
            $this->saveGeral($row,'infadicmapa');
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    } 
    
    /*
        Replica Dados Mês Anterior: Essa ação deve replicar todos os itens adicionais incluídos no mapa do mês anterior, 
        porém o campo Valor será zerado, para que o usuário digite apenas esse campo. Se existirem dois registros com a 
        mesma descrição, porém com a palavra INICIAL e FINAL, o valor “FINAL” do mapa anterior já será buscado 
        automaticamente para o valor “INICIAL” do mês atual.
    */
    public function replicarInformacoesAdicionais($row){
        try{    
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->factoryApuracaoDeImposto->deleteInfadicmapa(
                array(
                    "NRORG" => $this->nrorg,
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                )
            );
            $this->entityManager->flush();
            
            $informacoes = $this->factoryApuracaoDeImposto->getInformacoesAdiconaisMesAnterior(
                array(
                    "P_CDEMPRESA"    => $row['CDEMPRESA'],
                    "P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                    "P_CDIMPOSTO"    => $row["CDIMPOSTO"],
                    "P_CDINSCESTA"   => $row["CDINSCESTA"]
                )    
            );
            
            if($informacoes){
                foreach ($informacoes as $valor) {
                    $this->factoryApuracaoDeImposto->insertInfadicmapa(
                        array(
                            "P_NRORG"         => $this->nrorg,
                            "P_NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
                            "P_NRSEQINFADIC"  => $valor["NRSEQINFADIC"],
                            "P_CDINFADICMAPA" => $valor["CDINFADICMAPA"],
                            "P_DSINFADICMAPA" => $valor["DSINFADICMAPA"],
                            "P_VRINFADICMAPA" => $valor["VRINFADICMAPA"]
                        )    
                    );
                    $this->entityManager->flush();
                }
            }
            
            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    } 
    
    
    #############################################################
    #                                                           #
    #        Funções de apuração, bloqueio e desbloqueio        #
    #                                                           #
    #############################################################

    /*
        Função que ira buscar o mmapa referente ao outro imposto 
        corresponder de contribuições do mesmo periodos para realizar
        a apuração conjunta dos imposotos de contribuições
    */
    public function buscaMapaReferenteContribuicoes ($row){
        $avisos = [];
        $mapaReferente = $this->factoryApuracaoDeImposto->buscaMapaReferenteContribuicoes(
            array(
                "CDIMPOSTO"    => $row['CDIMPOSTOREFERENTE'],
                "CDEMPRESA"    => $row['CDEMPRESA'],
                "DTINIPERIODO" => $row['DTINIPERIODO'],
                "DTFIMPERIODO" => $row['DTFIMPERIODO'],
            )
        );
        // Não existe mapa Anterior como mesmo periodo
        if(empty($mapaReferente)){
            $ultimoMapaReferente = $this->factoryApuracaoDeImposto->buscaUltimoMapa(
                array(
                    "NRORG"       => $this->nrorg,
                    "CDIMPOSTO"   => $row["CDIMPOSTOREFERENTE"],
                    "CDEMPRESA"   => $row["CDEMPRESA"],
                    "CDINSCESTA"  => null,
                    "CDINSCMUNI"  => null,
                    "IDTIPOAPURA" => 'F'
                )    
            );
            if($ultimoMapaReferente){
                $dtFimUltimoMapa = $ultimoMapaReferente['DTFIMPERIODO'];
                $dtFimUltimoMapa = $this->formataData($dtFimUltimoMapa);
                
                $dtIniMapaAtual = $row['DTINIPERIODO'];
                $dtIniMapaAtual = $this->formataData($dtIniMapaAtual);
                
                $intervalo   = DateUtil::getQtdeDiasEntre($dtFimUltimoMapa,$dtIniMapaAtual);
                $nomeImposto =  $row['CDIMPOSTO'] ==='50' ?'PIS':'COFINS';
                
                // Verifica se o o ulitmo mapa termina um dia antes do periodo atual.
                if($intervalo ===1){
                    $mapa = new Row();
                    $mapa->set('__is_new',true);
                    $this->setCamposAuditoria($mapa);
                    $mapa->set('NRORG',$this->nrorg);
                    $mapa->set('CDEMPRESA',$row['CDEMPRESA']);
                    $mapa->set('CDIMPOSTO',$row['CDIMPOSTOREFERENTE']);
                    $mapa->set('DTINIPERIODO',$row['DTINIPERIODO']);
                    $mapa->set('DTFIMPERIODO',$row['DTFIMPERIODO']);
                    $mapa->set('NRSEQUMAPAPU',$this->geraProximoSequencialMapaApuracao());
                    $mapa->set('IDTIPOAPURA','F');
                    $mapa->set('CDINSCESTA',null);
                    $mapa->set('CDINSCMUNI',null);
                    $mapa->set('NMMAPAAPUR', str_ireplace($nomeImposto,$row['NMIMPOSTOREFERENTE'],$row['NMMAPAAPUR'])  );
                    $this->factoryApuracaoDeImposto->insertMapaapuracContribuicoes(
                        array(
                            "NRORG"        => $this->nrorg,
                            "CDEMPRESA"    => $mapa['CDEMPRESA'],
                            "CDIMPOSTO"    => $mapa['CDIMPOSTO'],
                            "DTINIPERIODO" => $mapa['DTINIPERIODO'],
                            "DTFIMPERIODO" => $mapa['DTFIMPERIODO'],
                            "NRSEQUMAPAPU" => $mapa['NRSEQUMAPAPU'],
                            "NMMAPAAPUR"   => $mapa['NMMAPAAPUR']
                        )
                    ); 
                    $this->entityManager->flush(); 
                    
                    $this->insertDetalhamento($mapa);
                    $this->entityManager->flush();
                    return $mapa;
                }else{
                    return 'MAPASEQ';
                }
                
            }
            //Caso não exista nenhum mapa, realiza a inserção do mapa referente
            else{
                $mapa = new Row();
                $mapa->set('__is_new',true);
                $this->setCamposAuditoria($mapa);
                $mapa->set('NRORG',$this->nrorg);
                $mapa->set('CDEMPRESA',$row['CDEMPRESA']);
                $mapa->set('CDIMPOSTO',$row['CDIMPOSTOREFERENTE']);
                $mapa->set('DTINIPERIODO',$row['DTINIPERIODO']);
                $mapa->set('DTFIMPERIODO',$row['DTFIMPERIODO']);
                $mapa->set('NRSEQUMAPAPU',$this->geraProximoSequencialMapaApuracao());
                $mapa->set('IDTIPOAPURA','F');
                $mapa->set('CDINSCESTA',null);
                $mapa->set('CDINSCMUNI',null);
                $mapa->set('NMMAPAAPUR','');
                $this->factoryApuracaoDeImposto->insertMapaapuracContribuicoes(
                    array(
                        "NRORG"        => $this->nrorg,
                        "CDEMPRESA"    => $mapa['CDEMPRESA'],
                        "CDIMPOSTO"    => $mapa['CDIMPOSTO'],
                        "DTINIPERIODO" => $mapa['DTINIPERIODO'],
                        "DTFIMPERIODO" => $mapa['DTFIMPERIODO'],
                        "NRSEQUMAPAPU" => $mapa['NRSEQUMAPAPU'],
                        "NMMAPAAPUR"   => $mapa['NMMAPAAPUR']
                    )
                ); 
                $this->entityManager->flush(); 
                
                $this->insertDetalhamento($mapa);
                $this->entityManager->flush();
                return $mapa;
            }
           
        }
        //Existe mapa Anterior como mesmo periodo
        else{
            // Caso o mapa esteja bloqueado não e possivel apurar 
            if($mapaReferente[0]['IDBLOQMAPAPU'] ==='S'){
                return 'MAPABLOQ';
            }else{
                $mapaReferente[0]['IDTIPOAPURA']= 'F';
                return $mapaReferente[0];
            }
        }
        

    }
    
    public function setaMapaApurado($row){
        $params = array(
            'nrorg' => $this->nrorg,
            'nrsequmapapu' => $row['NRSEQUMAPAPU']
        );
        $mapaapurac = $this->entityManager->getRepository(Mapaapurac::class)->findOneBy($params);
        // Seta o mapa como apurado (A).
        $mapaapurac->setIdbloqmapapu('A');
        $this->entityManager->persist($mapaapurac);
    }
    
    /*
        Valida a situações que impeçam de realizar apuração
        dos Mapas de Contribuições, quando reazalizada de maneira simultanea
    */
    public function validaApuracaoMapasContribuicoes ($row){
        $params = array(
            "NRORG" => $this->nrorg
        );
        $dataInicialNota = $this->factoryApuracaoDeImposto->buscaDataParametroFiscal($params);

        // A data deve estar parametrizada.
        if (!$dataInicialNota) {
            throw new \Exception("Deve ser informada na tela Parâmetros de Fiscal a data inicial para buscar notas fiscais fora do período de apuração.");
        }
        $dataInicialNota = $this->formataData($dataInicialNota);
        $dataMapa = $this->formataData($row['DTINIPERIODO']);
        // A data parametrizada deve vir antes da data do mapa.
        if (DateUtil::comparaDatas($dataInicialNota, $dataMapa, DateUtil::DEPOIS)) {
            throw new \Exception("Data inicial para verificação de notas fiscais fora do período deve ser menor que a data inicial do mapa de apuração.");
        }
    }
    
    /*
        Função que ira validar cadaq mapa individualmente 
        na apuração de simultanea de Impostos de Contribuições
    */
    public function validaMapaContribuicoes ($row){
        $mapaAnterior = $this->buscaMapaAnterior($row);
        //Valida se o Mapa Anterior estar apurado.
        if ( $mapaAnterior &&  $mapaAnterior['IDBLOQMAPAPU'] != 'S' && $mapaAnterior['IDBLOQMAPAPU'] != 'A'  )
            return false;
        $avisos =   $this->validaApuracaoContribuicoes($row, false);
        return $avisos;
    }
    
    /*
        Função para realizar apuração conjunta dos mapas do
        PIS E COFINS
    */
    public function apuraMapasContribuicoes($row) {
        try {
            $avisosMapaAtual     = [];
            $avisosMapaReferente = [];
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            // realiza Validação que impede a apuração dos dois mapas;
            $this->validaApuracaoMapasContribuicoes($row);
            
            $resulMapaAtual = $this->validaMapaContribuicoes($row);
            if($resulMapaAtual === false)
                $avisosMapaAtual = array('mensagens'=> [ 'Mapa anterior deve estar apurado.' ] , 'apurado'=>false);
            else{
                $this->callProcedureApurarMapa($row, false);
                $this->setaMapaApurado($row);
                $this->entityManager->flush();
                $avisosMapaAtual = array('mensagens'=> $resulMapaAtual , 'apurado'=>true);
            }
            
            $mapaContribuicoes = $this->buscaMapaReferenteContribuicoes($row);
            if($mapaContribuicoes ==='MAPASEQ'){
                $avisosMapaReferente = array('mensagens'=> [ 'Não foi possivel inserir um mapa sequencial referente a esse período.' ] , 'apurado'=>false);
                $connection->commit();
                return  array('mapaAtual' =>$avisosMapaAtual, 'mapaReferente'=>$avisosMapaReferente);
            }else if($mapaContribuicoes ==='MAPABLOQ'){
                $avisosMapaReferente = array('mensagens'=> [ 'Não foi possível apurar o mapa pois ele está bloqueado.' ] , 'apurado'=>false);
                $connection->commit();
                return  array('mapaAtual' =>$avisosMapaAtual, 'mapaReferente'=>$avisosMapaReferente);
            }else{
                $resulMapaReferente = $this->validaMapaContribuicoes($mapaContribuicoes);
                if($resulMapaReferente === false)
                    $avisosMapaReferente = array('mensagens'=> [ 'Mapa anterior deve estar apurado.' ] , 'apurado'=>false);
                else{
                    $this->callProcedureApurarMapa($mapaContribuicoes, false);
                    $this->setaMapaApurado($mapaContribuicoes);
                    $this->entityManager->flush();
                    $avisosMapaReferente = array('mensagens'=> $resulMapaReferente , 'apurado'=>true);
                }
                $connection->commit();
                return  array('mapaAtual' =>$avisosMapaAtual, 'mapaReferente'=>$avisosMapaReferente);    
            }
                
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    /*
     * Apura um mapa de apuração, se o anterior estiver apurado, dada a empresa.
     * Retorna quaisquer erros de validação não-bloqueantes.
     */
    public function apurarMapa($row, $bloquearMapa = false) {
        $avisos = $this->validaApuracao($row, $bloquearMapa);
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            $aviso = $this->callProcedureApurarMapa($row, $bloquearMapa);
            // Verifica se algum aviso foi retornado.
            if (!empty($aviso)) {
                $avisos[] = $aviso;
            }

            $params = array(
                'nrorg' => $this->nrorg,
                'nrsequmapapu' => $row['NRSEQUMAPAPU']
            );
            // Verifica se o mapa selecionado é de Simples Nacional.
            if ($this->isSimplesNacional($row)) {
                $currentTime = $this->environment->getCurrentTime();
                $mapaapurac = $this->entityManager->getRepository(Mapaapuracsn::class)->findOneBy($params);
                // Seta a data de realização da apuração para o Simples.
                $mapaapurac->setDtrealapura($currentTime);
            }
            // Se não for, busca da tabela geral.
            else {
                $mapaapurac = $this->entityManager->getRepository(Mapaapurac::class)->findOneBy($params);
            }
            // Seta o mapa como apurado (A).
            $mapaapurac->setIdbloqmapapu('A');
            $this->entityManager->persist($mapaapurac);
            $this->entityManager->flush();
            
            $mapaapurestadoArray   = $this->factoryApuracaoDeImposto->getTodosMapaapurestado(
                array(
                    "NRSEQUMAPAPU"     => $row["NRSEQUMAPAPU"],
                    "NRORG"            => $this->nrorg
                )    
            );
            
            foreach($mapaapurestadoArray as $mapaapurestado){
                $mapaapurestado['CDIMPOSTO'] = $row["CDIMPOSTO"];
                $mapaapurestado['IDTIPOAPURA'] = $row["IDTIPOAPURA"];
                
                $this->updateDetalhamento($mapaapurestado);
            }
            
            if($this->isImpostosIrpjCsll($row)){
                $params = array(
                    "NRORG" => $this->nrorg,
                    "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                );
                
                $this->factoryApuracaoDeImposto->deleteDemoctbtemp();
                
                $this->factoryApuracaoDeImposto->deletaRecebrutaliqmapa($params);
                
                $this->factoryApuracaoDeImposto->deletaItdetalheapur($params);
                
                $this->factoryApuracaoDeImposto->insertMapaapuimpofed($params);
                
                $this->entityManager->persist($mapaapurac);
                $this->entityManager->flush();
                
                $row['CDTIPODEMO'] = $this->factoryApuracaoDeImposto->adqAliqIRPJCSLL($params = array(
                                        "P_CDEMPRESA" => $row['CDEMPRESA'],
                                        "P_CDIMPOSTO" => $row["CDIMPOSTO"],
                                    ))['CDTIPODEMO'];
                                    
                $row['NRDRE'] = $this->factoryApuracaoDeImposto->adsDemonsOfic($row);
                
                $params = array(
                    "DTINICIAL"     => $row["DTINIPERIODO"],
                    "DTFINAL"       => $row["DTFIMPERIODO"],
                    "CDIMPOSTO"     => $row["CDIMPOSTO"],
                    "CDEMPRESA"     => $row["CDEMPRESA"],
                    "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
                    "GERBLCT"       => isset($row["GERBLCT"]) ? $row["GERBLCT"] : 'N',
                    "NRDRE"         => $row['NRDRE'],
                );

                if($row['CDIMPOSTO'] == '25') {
                    $this->factoryApuracaoDeImposto->apuraIRPJ($params);
                } else if($row['CDIMPOSTO'] == '70') {
                    $this->factoryApuracaoDeImposto->apuraCSLL($params);
                }
                
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }

        return $avisos;
    }

    /*
     * Apura e bloqueia um mapa de apuração, se o anterior estiver bloqueado,
     * dada a empresa.
     */
    public function bloquearMapa($row) {
        try{
            
            // Chama a função para apurar mapa passando um parâmetro que indica
            // que o mapa deve ser bloqueado após a apuração.
            
            // Faz a validação do bloqueio do mapa do imposto IMCS ST
            if( $row['CDIMPOSTO'] === self::IMPOSTO_ICMSST )
                $this->validaEstadosIcmsSt($row);
            
            $this->apurarMapa($row, true);
    
            // Executa validação específica para mapas de Contribuições ao bloquear
            // um mapa de apurações.
            if ($this->isContribuicoes($row)) {
                $this->validaArrecadacaoContribuicoes($row);
            }
    
            $params = array(
                'nrorg' => $this->nrorg,
                'nrsequmapapu' => $row['NRSEQUMAPAPU']
            );
            // Verifica se o mapa selecionado é de Simples Nacional.
            if ($this->isSimplesNacional($row)) {
                $mapaapurac = $this->entityManager->getRepository(Mapaapuracsn::class)->findOneBy($params);
            }
            // Se não for, busca da tabela geral.
            else {
                $mapaapurac = $this->entityManager->getRepository(Mapaapurac::class)->findOneBy($params);
            }
    
            // Seta o mapa como bloqueado (S).
            $mapaapurac->setIdbloqmapapu('S');
            $this->entityManager->persist($mapaapurac);
            $this->entityManager->flush();
        
        }
        catch (\Exception $e) {
            throw $e;
        }
    }

    /*
     * Função que chama a procedure de apuração de mapas e valida possíveis
     * erros retornados.
     */
    private function callProcedureApurarMapa($row, $bloquearMapa) {
        $params = array(
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "DTINIPERIODO"  => $row["DTINIPERIODO"],
            "DTFIMPERIODO"  => $row["DTFIMPERIODO"],
            "NRORG"         => $this->nrorg
        );
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $codigoRetorno = $this->factoryApuracaoDeImposto->apuraMapaSimples($params);
            if ($codigoRetorno == self::FAIXA_NAO_CADASTRADA) {
                throw new \Exception("Não há Faixa de Valores correspondente ao Valor do Faturamento dos últimos 12 meses (R$).");
            }
            else if ($codigoRetorno == self::DOCUMENTOS_FORA_PERIODO) {
                throw new \Exception("Na apuração existem documentos fora do período.");
            }
            else if ($codigoRetorno == self::TABELA_SIMPLES_NAO_CADASTRADA) {
                throw new \Exception("Não existe Tabela do Simples Nacional relacionada à Empresa.");
            }
        }
        // Verifica se o mapa é de Contribuições.
        else if ($this->isContribuicoes($row)) {
            $params = array(
                "CDIMPOSTONFE" => $row["CDIMPOSTO"],
                "NRORG" => $this->nrorg
            );
            // Retorna o parâmetro de imposto recuperável para o imposto do
            // mapa.
            $idimporecupnfe = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDIMPORECUPNFE"];
            $params = array(
                "DTINICIAL"         => $row["DTINIPERIODO"],
                "DTFINAL"           => $row["DTFIMPERIODO"],
                "IDIMPORECUPNFE"    => $idimporecupnfe,
                "CDIMPOSTONFE"      => $row["CDIMPOSTO"],
                "NRSEQUMAPAPU"      => $row["NRSEQUMAPAPU"],
                "CDEMPRESA"         => $row["CDEMPRESA"],
                "P_BOBLOQUEIA"      => $bloquearMapa
            );
            $this->factoryApuracaoDeImposto->apuraMapaContribuicoes($params);
        }
        else if($this->isInss($row)){
            $this->factoryApuracaoDeImposto->calculaInss(
                array(
                    
                    "P_VRBCFOLHA"      => $row["VRTTBACALDEB"],
                    "P_VRAPURFOLHA"    => $row["VRTTCTBLDEB"],
                    "P_CDEMPRESA"      => $row["CDEMPRESA"],
                    "P_CDIMPOSTONFE"   => $row["CDIMPOSTO"],
                    "P_NRSEQUMAPAPU"   => $row['NRSEQUMAPAPU'],
                    "P_DTINICIAL"      => $row["DTINIPERIODO"],
                    "P_DTFINAL"        => $row["DTFIMPERIODO"],
                    "P_IDCUMUPISCOFIL" => null
                )
            );
            $this->entityManager->flush();
            
            $this->factoryApuracaoDeImposto->calculaTotalInss(array("P_NRSEQUMAPAPU"=> $row['NRSEQUMAPAPU']));
            $this->entityManager->flush();
        }
        else {
            $inscricao = $row["IDTIPOAPURA"] == 'E' ? $row["CDINSCESTA"] : ($row["IDTIPOAPURA"] == 'M' ? $row["CDINSCMUNI"] : null);
            $params = array(
                "CDIMPOSTO"     => $row["CDIMPOSTO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDINSCRICAO"   => $inscricao,
                "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "BOBLOQ"        => $bloquearMapa ? 'S':'N'
            );

            $strRetorno = $this->factoryApuracaoDeImposto->apuraMapa($params);
            //Somente quando está apurando o ICMS ST pela primeira vez, gera as guias referentes
            if( $row['CDIMPOSTO'] === self::IMPOSTO_ICMSST ){
                if( empty( $this->factoryApuracaoDeImposto->verificaGuiaIcmsSt( array("P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]) ) ) )  
                    $this->factoryApuracaoDeImposto->gerarGuiaIcmsSt(array("NRSEQUMAPAPU"=>$row["NRSEQUMAPAPU"]));
            }
            // Verifica se foi retornado 'OK', indicando que não houve erros.
            if (substr($strRetorno, 0, 2) === 'OK') {
                // Se houver alguma nota pendente, haverá um 'N' após o 'OK',
                // indicando que a apuração ocorreu com sucesso, mas há notas
                // pendentes ou sem incidência para o mapa.
                if (substr($strRetorno, 2, 1) === 'S') {
                    return 'Existem notas fora do período na apuração.';
                }
            }
            // Se não houver o 'OK', retornará uma mensagem de erro.
            else {
                if ($bloquearMapa) {
                    throw new \Exception($strRetorno);
                }
                return $strRetorno;
            }
        }
    }

    /*
     * Desbloqueia um mapa de apuração, se ele for o último bloqueado, dado o
     * filtro.
     */
    public function desbloquearMapa($row) {
        // Verifica se o mapa está bloqueado.
        if ($row["IDBLOQMAPAPU"] != 'S') {
            throw new Exception("Mapa deve estar bloqueado.");
        }
        // Verifica se há uma preparação de pagamento para o mapa.
        if ($this->existePreparacaoPagamento($row)) {
            throw new \Exception("Mapa não pode ser desbloqueado. Foi realizada a preparação de pagamento para o imposto.");
        }
        $mapaPosterior = $this->buscaMapaPosterior($row);
        // Verifica se há um mapa posterior e se ele está bloqueado.
        if ($mapaPosterior && $mapaPosterior['IDBLOQMAPAPU'] == 'S') {
            throw new \Exception("Mapa posterior deve estar desbloqueado.");
        }
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $params = array(
                'nrorg' => $this->nrorg,
                'nrsequmapapu' => $row['NRSEQUMAPAPU']
            );
            // Verifica se o mapa selecionado é de Simples Nacional.
            if ($this->isSimplesNacional($row)) {
                $currentTime = $this->environment->getCurrentTime();
                $mapaapurac = $this->entityManager->getRepository(Mapaapuracsn::class)->findOneBy($params);
                // Seta a data de realização da apuração para o Simples.
                $mapaapurac->setDtrealapura($currentTime);
            }
            // Se não for, busca da tabela geral.
            else {
                $mapaapurac = $this->entityManager->getRepository(Mapaapurac::class)->findOneBy($params);
            }
            // Seta o mapa como apurado, uma vez que ele teve de ser apurado
            // antes de ser bloqueado.
            $mapaapurac->setIdbloqmapapu('A');
            $this->entityManager->persist($mapaapurac);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /*
     * Executa todas as validações necessárias em um mapa ao apurar. Algumas
     * validações não impedem a apuração, mas impedem o bloqueio, então o
     * parânetro "bloquearMapa" é verificado para decidir se uma exceção será
     * lançada ou se somente o erro vai ser retornado para exibição na tela.
     */
    private function validaApuracao($row, $bloquearMapa) {
        $mapaAnterior = $this->buscaMapaAnterior($row);
        // Valida se o mapa anterior foi previamente apurado ou bloqueado, caso
        // a apuração seja para bloquear o mapa.
        if ($mapaAnterior && $mapaAnterior['IDBLOQMAPAPU'] != 'S' && ($mapaAnterior['IDBLOQMAPAPU'] != 'A' || $bloquearMapa)) {
            throw new \Exception("Mapa anterior deve estar " . ($bloquearMapa ? "bloqueado." : "apurado."));
        }

        $params = array(
            "NRORG" => $this->nrorg
        );
        $dataInicialNota = $this->factoryApuracaoDeImposto->buscaDataParametroFiscal($params);
        // A data deve estar parametrizada.
        if (!$dataInicialNota) {
            throw new \Exception("Deve ser informada na tela Parâmetros de Fiscal a data inicial para buscar notas fiscais fora do período de apuração.");
        }
        $dataInicialNota = $this->formataData($dataInicialNota);
        $dataMapa = $this->formataData($row['DTINIPERIODO']);
        // A data parametrizada deve vir antes da data do mapa.
        if (DateUtil::comparaDatas($dataInicialNota, $dataMapa, DateUtil::DEPOIS)) {
            throw new \Exception("Data inicial para verificação de notas fiscais fora do período deve ser menor que a data inicial do mapa de apuração.");
        }

        if ($this->isContribuicoes($row)) {
            return $this->validaApuracaoContribuicoes($row, $bloquearMapa);
        }
        // Não executa validações extras para o Simples Nacional.
        else if ($this->isSimplesNacional($row)) {
            // Retorna um array vazio.
            return array();
        }
        
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $this->nrorg
        );
        $formaApuracao = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDFORMAPURNFE"];

        // Inicializa a variável de avisos que não bloqueiem a apuração.
        $avisos = array();

        // Verifica se o mapa é municipal para realização de validações de
        // incidência que não são consideradas em apurações municipais.
        if ($row["IDTIPOAPURA"] != 'M') {
            $params = array(
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDIMPOSTO"     => $row["CDIMPOSTO"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "NRORG"         => $this->nrorg,
            );
            // Verifica se há alguma nota fiscal pendente de aprovação ou sem
            // incidência de imposto.
            $notaPendente = $this->factoryApuracaoDeImposto->getNotaPendente($params);
            if ($notaPendente) {
                if ($bloquearMapa) {
                    throw new \Exception("Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.");
                }
                $avisos[] = 'Existem notas fiscais pendentes de aprovação/emissão ou sem incidência do imposto, favor verificar o relatório "Nota Fiscal Pendente" na tela Conferência Fiscal.';
            }
            $params = array(
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "NRORG"         => $this->nrorg,
            );
            // Realiza verificação de venda pendente de aprovação ou sem incidência
            // de imposto.
            $vendaPendente = $this->factoryApuracaoDeImposto->getVendaPendente($params);
            if ($vendaPendente) {
                if ($bloquearMapa) {
                    throw new \Exception("Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.");
                }
                $avisos[] = 'Existem vendas pendentes de aprovação/emissão ou sem incidência do imposto.';
            }
        }
        return $avisos;
    }

    /*
     * Executa as validações que possuem particularidades para mapas de apuração
     * de Contribuições.
     */
    private function validaApuracaoContribuicoes($row, $bloquearMapa) {
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $row["CDIMPOSTO"]
        );
        $formaApuracao = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDFORMAPURNFE"];

        // Inicializa a variável de avisos que não bloqueiem a apuração.
        $avisos = array();

        $params = array(
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "CDIMPOSTONFE"  => $row["CDIMPOSTO"],
            "DTFINFORPER"   => $row["DTINIPERIODO"],
            "IDFORMAPURNFE" => $formaApuracao,
            "NRORG"         => $this->nrorg,
        );
        $notaFiscal = $this->factoryApuracaoDeImposto->getNotaForaPeriodoContribuicoes($params);
        // Verifica se há alguma nota fiscal fora do período de apuração.
        if ($notaFiscal) {
            $errorMessage = "Existem notas fora do período na apuração.";
            $avisos[] = $errorMessage;
        }

        $params = array(
            "NRSEQUMAPAPU"      => $row["NRSEQUMAPAPU"],
            "DTFINFORPER"       => $row["DTINIPERIODO"],
            "IDFORMAPURNFE"     => $formaApuracao,
            "NRORG"             => $this->nrorg,
        );
        // Verifica se há alguma nota fiscal de compensação fora do período de
        // apuração.
        $notaCompensacao = $this->factoryApuracaoDeImposto->getNotaCompensacaoForaPeriodo($params);
        if ($notaCompensacao) {
            $errorMessage = "Existem notas de compensação fora do período na apuração.";
            $avisos[] = $errorMessage;
        }

        $params = array(
            "DTINICIAL"     => $row["DTINIPERIODO"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "DTFINAL"       => $row["DTFIMPERIODO"],
            "CDIMPOSTO"     => $row["CDIMPOSTO"],
            "IDFORMAPUR"    => $formaApuracao,
            "NRORG"         => $this->nrorg
        );
        // Verifica se há alguma nota fiscal pendente de aprovação ou sem
        // incidência de imposto.
        $notaPendente = $this->factoryApuracaoDeImposto->getNotaPendenteContribuicoes($params);
        if ($notaPendente) {
            if ($bloquearMapa) {
                throw new \Exception("Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.");
            }
            $avisos[] = 'Existem notas fiscais pendentes de aprovação/emissão ou sem incidência do imposto, favor verificar o relatório "Nota Fiscal Pendente" na tela Conferência Fiscal.';
        }

        $params = array(
            "DTINICIAL" => $row["DTINIPERIODO"],
            "DTFINAL"   => $row["DTFIMPERIODO"],
            "CDEMPRESA" => $row["CDEMPRESA"],
            "NRORG"     => $this->nrorg,
        );
        // Realiza verificação de venda pendente de aprovação ou sem incidência
        // de imposto.
        $vendaPendente = $this->factoryApuracaoDeImposto->getVendaPendenteContribuicoes($params);
        if ($vendaPendente) {
            if ($bloquearMapa) {
                throw new \Exception("Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.");
            }
            $avisos[] = 'Existem vendas pendentes de aprovação/emissão ou sem incidência do imposto.';
        }
        return $avisos;
    }

    /*
     * Validação de Arrecadação Fiscal para mapas de Contribuições quando há
     * um valor de impostos a recolher.
     */
    private function validaArrecadacaoContribuicoes($row) {
        $params = array(
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
            "NRORG" => $this->nrorg
        );
        $mapaapuracDetalhamento = $this->factoryApuracaoDeImposto->getMapaDetalhamentoContribuicoes($params);
        // Verifica se há detalhamento inserido. Este erro só ocorrerá em casos
        // extremos, onde o próprio sistema não cadastrou o detalhamento para o
        // mapa.
        if ($mapaapuracDetalhamento == null) {
            throw new \Exception("Erro no cadastramento do detalhamento do mapa de apuração.");
        }

        // Verifica se há imposto a recolher.
        if ($mapaapuracDetalhamento["VRTOTAIMPOS"] > 0) {
            // Data de vencimento da guia não pode ser nulo.
            if ($mapaapuracDetalhamento["DTVENCGUIA"] == null) {
                throw new \Exception("Data de Vencimento da guia deve ser informada.");
            }
            // Se houver débito cumulativo, o código da receita cumulativo não
            // pode ser nulo.
            if ($mapaapuracDetalhamento["VRDEBITOSCUM"] > 0 && $mapaapuracDetalhamento["CDRECECUMU"] == null) {
                throw new \Exception("Código da Receita Cumulativo deve ser informado.");
            }
            // Se houver débito não cumulativo, o código da receita não
            // cumulativo não pode ser nulo.
            if ($mapaapuracDetalhamento["VRDEBITOSNC"] > 0 && $mapaapuracDetalhamento["CDRECENCUMU"] == null) {
                throw new \Exception("Código da Receita Não Cumulativo deve ser informado.");
            }
        }
    }

    /*
        Todo Estado da tabela MAPAAPURESTADO deve possuir o  valor
        do imposto a recolher  igual a soma de todas as receitas na aba
        de arrecadação fiscal
    */
    public function validaEstadosIcmsSt ($row){
        try{
            $params = array(
                "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                "NRORG" => $this->nrorg
            );
            
            $arrayEstados   = [];
            $mapaapurestado = $this->factoryApuracaoDeImposto->getTodosMapaapurestado($params);
            if(!empty ($mapaapurestado) ){
                foreach ($mapaapurestado as $estado) {
                    $somaValorReceita = $this->factoryApuracaoDeImposto->getSomaArrecadacaoFiscalIcmsSt(
                        array(
                            "NRSEQUMAPAPU"     => $estado["NRSEQUMAPAPU"],
                            "CDPAIS"           => $estado["CDPAIS"],
                            "SGESTADO"         => $estado["SGESTADO"],
                            "NRSEQMAPAAPUREST" => $estado["NRSEQMAPAAPUREST"],
                        )    
                    );
                    
                    if($somaValorReceita['VALORRECEITA'] !== $estado['VRIMPARECOLHER'])
                        $arrayEstados [] = $estado['SGESTADO'];
                }   
            }
            
            if( count($arrayEstados)>0)
                throw new \Exception("O(s) seguinte(s) estado(s) está(ão) como o valor da guia de arrecadação diferente do imposto a recolher: ".implode(',',$arrayEstados));


        }
        catch (\Exception $e) {
            throw $e;
        }
        
    }

    #############################################################
    #                                                           #
    #                     Funções auxiliares                    #
    #                                                           #
    #############################################################

    /*
     * Aplica máscara de inscrição (CNPJ ou CPF) nos registros buscados.
     */
    public function aplicarMascaraInscricao($rows) {
        foreach ($rows as &$row) {
            if (strlen($row['CNPJ']) == 14) {
                $row['CNPJ'] = $this->masks->mask($row['CNPJ'], '##.###.###/####-##');
            }
            else if (strlen($row['CNPJ'])==11) {
                $row['CNPJ'] = $this->masks->mask($row['CNPJ'], '###.###.###-##');
            }
        }
        return $rows;
    }

    /*
     * Busca o mapa anterior ao selecionado, conforme parâmetros associados.
     */
    private function buscaMapaAnterior($row) {
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $params = array(
                "NRORG" => $this->nrorg,
                "CDEMPRESA" => $row['CDEMPRESA'],
                "DTINIPERIODO" => $row['DTINIPERIODO']
            );
            $mapaAnterior = $this->factoryApuracaoDeImposto->buscaMapaAnteriorSimples($params);
        }
        else {
            $params = array(
                "NRORG"         => $this->nrorg,
                "CDEMPRESA"     => $row['CDEMPRESA'],
                "CDIMPOSTO"     => $row['CDIMPOSTO'],
                "CDINSCESTA"    => $row['CDINSCESTA'],
                "CDINSCMUNI"    => $row['CDINSCMUNI'],
                "IDTIPOAPURA"   => $row['IDTIPOAPURA'],
                "DTINIPERIODO"  => $row['DTINIPERIODO']
            );
            $mapaAnterior = $this->factoryApuracaoDeImposto->buscaMapaAnterior($params);
        }
        return $mapaAnterior;
    }

    /*
     * Busca o mapa posterior ao selecionado, conforme parâmetros associados.
     */
    public function buscaMapaPosterior($row) {
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $params = array(
                "NRORG" => $this->nrorg,
                "CDEMPRESA" => $row['CDEMPRESA'],
                "DTINIPERIODO" => $row['DTINIPERIODO']
            );
            $mapaPosterior = $this->factoryApuracaoDeImposto->buscaMapaPosteriorSimples($params);
        }
        else {
            $params = array(
                "NRORG"         => $this->nrorg,
                "CDEMPRESA"     => $row['CDEMPRESA'],
                "CDIMPOSTO"     => $row['CDIMPOSTO'],
                "CDINSCESTA"    => $row['CDINSCESTA'],
                "CDINSCMUNI"    => $row['CDINSCMUNI'],
                "IDTIPOAPURA"   => $row['IDTIPOAPURA'],
                "DTINIPERIODO"  => $row['DTINIPERIODO']
            );
            $mapaPosterior = $this->factoryApuracaoDeImposto->buscaMapaPosterior($params);
        }
        return $mapaPosterior;
    }

    /*
     * Busca a próxima data inicial de um mapa a ser inserido e seu código
     * sequencial. Retorna uma data sugerida na parametrização fiscal caso não
     * haja um mapa anterior para o filtro selecionado.
     */
    public function buscaPrxDataIniECodigoMapa($row) {
        $result = array();

        // Busca a data inicial do próximo mapa a ser inserido, que deve ser
        // sequencial ao último mapa existente.
        $dtInicial = $this->factoryApuracaoDeImposto->getPrxDtIniMapa(array(
            "NRORG"         => $this->nrorg,
            "CDEMPRESA"     => $row['CDEMPRESA'],
            "CDIMPOSTO"     => $row['CDIMPOSTO'],
            "CDINSCESTA"    => $row['CDINSCESTA'],
            "CDINSCMUNI"    => $row['CDINSCMUNI'],
            "IDTIPOAPURA"   => $row['IDTIPOAPURA']
        ));

        // Se ainda não existirem mapas para o imposto, sugere a data
        // parametrizada como data inicial para apuração de imposto na tela
        // Parâmetros de Fiscal. No retorno também deve ser informado que é o
        // primeiro mapa, pois somente nessa situação o usuário poderá alterar
        // a data inicial.
        $result["IDPRIMEIROMAPA"] = !$dtInicial;
        if (!$dtInicial) {
            $dtInicial = $this->factoryApuracaoDeImposto->getDtIniApuracao(array(
                "NRORG" => $this->nrorg
            ));
        }
        $result["DTINIPERIODO"] = $dtInicial;
        $result["NRSEQUMAPAPU"] = $this->geraProximoSequencialMapaApuracao();
        return $result;
    }
    /*
     * Verifica se o mapa selecionado é o último mapa, dado o filtro.
     */
    public function checkUltimoMapa($row) {
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $params = array(
                "NRORG" => $this->nrorg,
                "CDEMPRESA" => $row["CDEMPRESA"],
            );

            $ultimoMapa = $this->factoryApuracaoDeImposto->buscaUltimoMapaSimples($params);
        }
        else {
            $params = array(
                "NRORG"         => $this->nrorg,
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDIMPOSTO"     => $row["CDIMPOSTO"],
                "CDINSCESTA"    => $row['CDINSCESTA'],
                "CDINSCMUNI"    => $row['CDINSCMUNI'],
                "IDTIPOAPURA"   => $row['IDTIPOAPURA']
            );

            $ultimoMapa = $this->factoryApuracaoDeImposto->buscaUltimoMapa($params);
        }
        // Retorna true caso o mapa selecionado seja o último.
        return $ultimoMapa["NRSEQUMAPAPU"] == $row["NRSEQUMAPAPU"];
    }

    /*
     * Verifica se um sequencial inserido para um item de detalhamento de um
     * mapa de apuração já está sendo utilizado por outro item.
     */
    public function checkValidSequencialDetalhamento($row) {
        $params = array(
            "NRORG"         => $this->nrorg,
            "NRSEQITDETAPU" => $row["NRSEQITDETAPU"],
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "IDTIPODETAPU"  => $row["IDTIPODETAPU"]
        );
        $item = $this->factoryApuracaoDeImposto->getItemDetalhamento($params);
        // Retorna true se não existir outro registro com o sequencial
        // informado.
        return empty($item);
    }

    /*
     * Verifica se existe uma preparação de pagamento para um mapa de apuração
     * que não seja de Simples Nacional.
     */
    private function existePreparacaoPagamento($row) {
        $preparacaoExists = false;
        // Verifica se o mapa não é de Simples Nacional
        if ($row["IDTIPOAPURA"] != 'S') {
            $params = array(
                "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"],
                "NRORG" => $this->nrorg
            );
            $preparacaoPagamento = $this->factoryApuracaoDeImposto->getPreparacaoPagamento($params);
            // Atribui true se retornar algum dado.
            $preparacaoExists = !!$preparacaoPagamento;
        }
        return $preparacaoExists;
    }

    /*
     * Formata uma data para 'DD/MM/YYYY', removendo hora, se necessário.
     * Retorna null se não for passado um parâmetro válido.
     */
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    /*
     * Retorna o próximo código sequencial para mapas de apuração válido.
     */
    private function geraProximoSequencialMapaApuracao() {
        return $this->newCode->getCodeByTableName('MAPAAPURACAO', $this->nrorg);
    }

    /*
     * Função para buscar o próximo código sequencial para um ajuste em um mapa
     * de apuração de Contribuições.
     */
    public function getNextSequencialAjusteContribuicoes($row) {
        $params = array(
            "NRORG" => $this->nrorg,
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
        );
        $nextSequencial = $this->factoryApuracaoDeImposto->getNextSequencialAjusteContribuicoes($params);
        // Adiciona zeros ao início do código até que ele tenha tamanho 10.
        for ($length = strlen($nextSequencial); $length < 10; $length++) {
            $nextSequencial = "0" . $nextSequencial;
        }
        return $nextSequencial;
    }

    /*
     * Verifica se um mapa é de apuração de Contribuições.
     */
    public  function isContribuicoes($row) {
        return in_array($row["CDIMPOSTO"], self::IMPOSTOS_CONTRIBUICOES);
    }
    
    /*
     * Verifica se um mapa é de apuração do IRPJ CSLL.
     */
    public  function isImpostosIrpjCsll($row) {
        return in_array($row["CDIMPOSTO"], self::IMPOSTOS_IRPJCSLL);
    }
    
     /*
     * Verifica se um mapa é de apuração de Impostos Gerais
     */
    public function isImpostosGerais($row) {
        $todosImpostos = [];
        $todosImpostos = array_merge(self::IMPOSTOS_CONTRIBUICOES,[self::IMPOSTOS_INSS], self::IMPOSTOS_IRPJCSLL);
        if($row['IDTIPOAPURA']==="S")
            return false;
        
        else
            //caso não seja nenhum dos impostos informados, significa que é Apuração de Impostos Gerais
            return (!in_array ($row["CDIMPOSTO"], self::IMPOSTOS_CONTRIBUICOES) );    
        
    }

    /*
     * Verifica se um mapa é de apuração de Contribuições.
     */
    public function isSimplesNacional($row) {
        return $row["IDTIPOAPURA"] == 'S';
    }
    
    /*
     * Verifica se um mapa é de apuração do INSS
     */
    public function isInss($row) {
        return $row["CDIMPOSTO"] == self::IMPOSTOS_INSS;
    }
    
    /*
     * Valida impostos relacionados com impostos NF-e, verificando se seus
     * atributos são iguais.
     */
    public function validaImpostosRelacionados($cdimpostoNfe) {
        // Busca os dados do Imposto NF-e.
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $cdimpostoNfe
        );
        $impostoNfe = $this->factoryApuracaoDeImposto->getImpostoNfe($params);

        // Busca os impostos relacionados ao código de imposto NF-e.
        $impostosRelac = $this->factoryApuracaoDeImposto->getImpostoRelac($params);

        // Se as duas tabelas possuem dados, em um primeiro momento a validação
        // ocorreu com sucesso.
        if (($impostosRelac) && ($impostoNfe)) {
            $paramRelacionamento = true;
        }
        else {
            $paramRelacionamento = false;
        }

        // Caso encontre algum imposto onde haja divergência na parametrização,
        // a validação deve retornar falso, para permitir o tratamento no
        // frontend.
        if ($paramRelacionamento) {
            foreach ($impostosRelac as $imposto) {
                if (($imposto['IDORGARRECAD'] != $impostoNfe['IDORGARRECADNFE']) ||
                    ($imposto['IDFORMAPUR']   != $impostoNfe['IDFORMAPURNFE'])   ||
                    ($imposto['IDIMPORECUP']  != $impostoNfe['IDIMPORECUPNFE'])) {
                    $paramRelacionamento = false;
                }
            }
        }

        $validation = array(
            "VDPARAMIMPRELAC" => $paramRelacionamento
        );
        return $validation;
    }



    public function validaImpostoApurac($cdimposto){
        
        // Busca os dados do Imposto
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTO" => $cdimposto
        );
         $imposto = $this->factoryApuracaoDeImposto->getImpostoApurac($params);
    }
    
    
    /*
     * Verifica se um mapa de apuração está bloqueado, lançando exceção em caso
     * positivo. Essa função deve ser chamada em todas as situações em que um
     * procedimento não pode ser executado com o mapa bloqueado.
     */
    public function validaMapaBloqueado($row) {
        $params = array(
            "NRORG" => $this->nrorg,
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
        );
        $idbloqmapapu = $this->factoryApuracaoDeImposto->getIdbloqmapapu($params);
        if ($idbloqmapapu == 'S') {
            throw new \Exception("Operação Bloqueada! Mapa de apuração está bloqueado.");
        }
    }
    
    public function geraGiaRs($row) {
         try{
             
             $this->factoryApuracaoDeImposto->geraGiaRs($row);
             
         } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function updateAjusteApurContr($row) {
         try{
            
            $params = array(
                'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                'NRSEQAJUSTE' =>$row['NRSEQAJUSTE'],
                'NRORG' =>$row['NRORG'] 
            );
            
            $exist = $this->factoryApuracaoDeImposto->verifyUpdateAjusteApurContr($params);
            if(!empty($exist)){ 
                $params = array(
                    'DTAJUSTE' => $row['DTAJUSTE'],
                    'IDTPAJUSTE' => $row['IDTPAJUSTE'],
                    'CDAJUSTE' => $row['CDAJUSTE'],
                    'CDAJUSTEBC' => $row['CDAJUSTEBC'],
                    'NRPROCESSO' => $row['NRPROCESSO'],
                    'DSAJUSTE' => $row['DSAJUSTE'],
                    'VRAJUSTE' => $row['VRAJUSTE'],
                    'CDTPCREDPICOITE' => $row['CDTPCREDPICOITE'],
                    'IDCUMUPISCOFIL' => $row['IDCUMUPISCOFIL'],
                    'IDINDICAJUSTE' => $row['IDINDICAJUSTE'],
                    'CDSITUTRIBITNFE' => $row['CDSITUTRIBITNFE'],
                    'VRBASECALCAJ' => $row['VRBASECALCAJ'],
                    'CDCONTCTBL' => $row['CDCONTCTBL'],
                    'CDFILIAL' => $row['CDFILIAL'],
                    'IDAPROPRIACAO' => $row['IDAPROPRIACAO'],
                    'NRRECIBO' =>$row['NRRECIBO'],
                    'NRSEQUMAPAPU' =>$row['NRSEQUMAPAPU'],
                    'NRSEQAJUSTE' =>$row['NRSEQAJUSTE'],
                    'NRORG' =>$row['NRORG'],
                    'CDCONTRATEIO' =>$row['CDCONTRATEIO']?$row['CDCONTRATEIO'] : null,
                    'CDEMPRESA'    =>$row['CDEMPRESA']?$row['CDEMPRESA'] : null,
                    'CDLOTE'       =>$row['CDLOTE'] ?$row['CDLOTE'] : null
                );
                $this->factoryApuracaoDeImposto->updateAjusteApurContr($params);
            }
             
         } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    private function setCamposAuditoria ($row){
        if($row['__is_new']){
            $row['NRORG']          = $this->nrorg;
            $row['IDATIVO']        = 'S';
            $row['DTINCLUSAO']     = $this->currentTime;
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGINCLUSAO']  = $this->nrorg;
            $row['CDOPERINCLUSAO'] = $this->operador;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;
        }else{
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;    
        }
    }

}
